* ------------------------------------------------------------------- SMC2Jerk
      Program SMC2Jerk


* Produces files of jerk (times dt) from acceleration file

* Input from a control file; here is an example:
*!Control file for program SMC2Jerk
*!Name of summary file:
* smc2jerk.sum
*!Character string to append to name of input file for output
* .jerk
*!List of files 
* t0781pea.smc
* t0781ped.smc
*Stop

* Dates: 01/27/05 - Written by D.M. Boore, patterned after SMC2VD
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
!        10/06/17 - Allow longer input file names

      character precision*8

 
c
c Dimension and declaration statements:

      real a(:), jerk(:)
      allocatable :: a, jerk 

      integer npts, nc_f_a 
      real tend
      real*8 cum_avg

      character date_c*8, time_c*11, ext4out*20

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)

      character buf*400, buf_upper*4, 
     :        f_sum*400, f_a*400, f_out*400

      character f_ctl*40
      logical f_ctl_exist, f_a_exist, rmv_trnd, use_v0_d0

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMC2Jerk.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2jerk.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2Jerk:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      
      ext4out = ' '
      read(nu_ctl,'(a)') ext4out
      call trim_c(ext4out,nc_ext4out)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

       
1000  continue
      buf = ' '
      read(nu_ctl,'(a)',end = 9999) buf
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
      if(buf_upper(1:4) .eq. 'STOP') go to 9999
      
      f_a = ' '
      f_a = buf(1:nc_buf)
      nc_f_a = nc_buf     
      inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
 
      if (.not. f_a_exist) then
          write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Passed tests; proceed

 
      call smc_npts(f_a(1:nc_f_a), npts, sps)

 
      allocate( a(npts+5), jerk(npts+5) )

* Now read the data from the files:

      write(*,'(1x,a)') 
     :   'Processing file:', f_a(1:nc_f_a)

      tskip = 0.0
      tlength = -1.0

      call SMCRead(f_a(1:nc_f_a), tskip, tlength,
     :               a,  
     :               char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      rnull = real_head(1)
 
      sps = real_head(2)
      ncomments_a = int_head(16)
      npts = int_head(17)
      dt = 1.0/sps
      tend = float(npts-1)*dt

 
* 1. Compute jerk

      jerk(1) = 0.0
      do i = 2, npts
        jerk(i) = (a(i) - a(i-1))
*        jerk(i) = (a(i) - a(i-1))/dt
      end do

 
* 2. Write output:

      f_out = ' ' 
      f_out = f_a(1:nc_f_a)//ext4out(1:nc_ext4out)
      call trim_c(f_out, nc_f_out)

        call datetime(date_c, time_c, iflag)

        ncomments = ncomments_a
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| Jerk, from processing acc file '//f_a(1:nc_f_a)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| using program SMC2Jerk'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c

  
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments 
        itype = 0


        call SMCWrite(f_out, precision,
     :                jerk, char_head, itype, 
     :                    int_head, real_head, comments)

  
      deallocate (a, jerk)

      go to 1000

9999  continue

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2Jerk


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\datetime.for'
      include '\forprogs\distaz.for'
 