* ------------------------------------------------------------------- SMC2FAS
      Program SMC2FAS


c Computes the Fourier amplitude spectra for specified time series.  
* The program reads a list of time series, and an output 
* file containing the spectra is written.
* The intended use is to provide input to
* CoPlot on the PC.

* Assumptions:

*    * With one exception, no assumption is made about equality of dt or npts
*      for each time series when computing spectra.  The exception is 
*      if df_intrp = 0, in which case the output frequencies are the same as 
*      the fft frequencies (within the bounds specified by the low and high 
*      interpolation parameters), and these must be the same for each spectrum
*      written to a file, since I assume a common frequency axis (although I 
*      could write separate frequency columns for each spectrum).

* Input from a control file; here is an example:
*Control file for SMC2FAS      ! first line
*Name of summary file:
* smc2fas.sum
*Name of output file:
* smc2fas.asc
*For each file: f_smc/idtag/tskip,tlength,frctnfrtaper/df_intrp, etc (12 max):
* \psv\taiwan99\t0781pea.smc
*   t0781pea                      ! up to 8 characters, starting in any column
*    0.0 20.0 0.05 ! tskip, tlength, frctnfrtaper
*    0.0 0.0 50.0  ! df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high 
*    0.0 -1.0     ! df_smooth(Hz;0=none), signnpw2(<0, backup for npw2, no zpad)
*    dc_remove? y
* \psv\taiwan99\t1293l3v.smc
*   t1293l3v                      ! up to 8 characters, starting in any column
*    10.0 20.0 0.05 ! tskip, tlength, frctnfrtaper
*    0.0 0.0 50.0  ! df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high 
*    0.0 -1.0     ! df_smooth(Hz;0=none), signnpw2(<0, backup for npw2, no zpad)
*    dc_remove? y
* \psv\taiwan99\t0781ped.smc
*   t0781ped                      ! up to 8 characters, starting in any column
*    20.0 20.0 0.05 ! tskip, tlength, frctnfrtaper
*    0.0 0.0 50.0  ! df_intrp(0=FFT freqs), f_intrp_low, f_intrp_high 
*    0.0 -1.0     ! df_smooth(Hz;0=none), signnpw2(<0, backup for npw2, no zpad)
*    dc_remove? y
*stop


* Dates: 12/12/00 - Written by D.M. Boore, patterned after TS2FAS and SMC2ASC
*        07/17/03 - Add sps to smc_npts parameter list
*        10/16/07 - Transparent change to user needed because smcread can
*                   read two input formats

c
c Dimension and declaration statements:

      parameter(nrec_max = 12)

      real fas(:), fas_out(:,:), freq(:), freq_out(:,:), ts(:)
      allocatable :: fas, fas_out, freq, freq_out, ts

      integer npts(nrec_max), nfreq_out(nrec_max), nc_f_smc(nrec_max)
      real tskip(nrec_max), tlength(nrec_max), frctnfrtaper(nrec_max)
      real df_intrp(nrec_max), f_intrp_low(nrec_max), 
     :  f_intrp_high(nrec_max)
      real df_smooth(nrec_max), signnpw2(nrec_max)
      real t_smc(nrec_max), t_fft(nrec_max)

      character char_head(11)*80, comments(500)*80
      real real_head(50)
      integer int_head(48)

      character buffer*70, cbuf*1,
     :        f_smc(nrec_max)*40, colhead(nrec_max)*12, idtag*80,
     :        bigbuf*255,  
     :        f_asc*40, f_sum*40


      character f_ctl*40
      logical f_ctl_exist, f_smc_exist, dc_remove(nrec_max)

c Date and time stamp:

*      call date(bufd)
*      call time(buft)

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') stop
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skip(nu_ctl,2)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      call skip(nu_ctl,1)
      f_asc = ' '
      read(nu_ctl,'(a)') f_asc
      write(nu_sum,'(a)') ' Output of program SMC_FAS:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl
      write(nu_sum,'(2a)') '   Output file = ', f_asc

            
      buffer = ' '
      read (nu_ctl, '(a)') buffer
      write(nu_sum,'(a)')  buffer
      
      nrec = 0

1000  continue
      
      buffer = ' '
      read (nu_ctl, '(a)') buffer
      write(nu_sum,'(a)')  buffer
      call upstr(buffer)
      call trim_c(buffer, nc_buf)
      if (buffer .eq. 'STOP') goto 2000
* Check for nrec > max number
        if (nrec + 1 .gt. nrec_max) then
           write(*,*)
           write(*, '(a   )') 
     :    ' nrec+1 > nrec_max; ignoring remaining records'
           write(*,*)
           go to 2000
        end if

      f_smc(nrec+1) = ' '
      f_smc(nrec+1) = buffer(1:nc_buf)

      inquire(file=f_smc(nrec+1), exist=f_smc_exist)
      if (.not. f_smc_exist) then
          write(*,'(3a)') ' ****** FILE ', f_smc(nrec+1), 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          go to 1000
      end if

* Passed tests; proceed

      nrec = nrec+ 1
      call trim_c(f_smc(nrec),nc_f_smc(nrec))

* Read idtag:
      buffer = ' '
      read(nu_ctl, '(a)') buffer
      call RCC (1,buffer,70,idtag,nc_id,ibgn,iend)
      if (nc_id .gt. 8) nc_id = 8
      write(nu_sum,'(2a,1x,i5)') 
     :  '  idtag, nc_id = ', idtag(1:nc_id), nc_id
        
* Read tskip, tlength, frctnfrtaper:
      read(nu_ctl,*) 
     :   tskip(nrec), tlength(nrec), frctnfrtaper(nrec)
      write(nu_sum,'(a)') 
     : ' tskip(nrec), tlength(nrec), frctnfrtaper(nrec) ='
      write(nu_sum,*) 
     :   tskip(nrec), tlength(nrec), frctnfrtaper(nrec)

      read (nu_ctl, *) df_intrp(nrec), f_intrp_low(nrec), 
     :                 f_intrp_high(nrec)
      read (nu_ctl, *) df_smooth(nrec), signnpw2(nrec)

      buffer = ' '
      read(nu_ctl, '(a)') buffer
      call RCC (2,buffer,70,cbuf,nc_cb,ibgn,iend)

      write(nu_sum, '(a)') 
     :    ' df_intrp, f_intrp_low, f_intrp_high, df_smooth, signnpw2='

      write(nu_sum, *) 
     :    df_intrp(nrec), f_intrp_low(nrec), f_intrp_high(nrec)
      write(nu_sum, *) 
     :    df_smooth(nrec), signnpw2(nrec)

      call upstr(cbuf)
      if (cbuf .eq. 'Y') then
        dc_remove(nrec) = .true.
      else
        dc_remove(nrec) = .false.
      end if
      write(nu_sum,'(a,1x,l1)')  ' dc_remove = ', dc_remove(nrec)
* Set up the column headings:
      colhead(nrec)(1:12) = '............'
      colhead(nrec)(1:4+nc_id) = 'fas:'//idtag(1:nc_id)
      
* Get number of points on this loop:

      call smc_npts(f_smc(nrec), npts(nrec), sps)

      go to 1000  ! loop back for another file

2000  continue

      close(nu_ctl)

* Figure out largest value of npts and allocate the array:
      call imnmax(npts,1,nrec,1,imin,imax) 
      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') ' imin, imax = ', imin, imax
      write(nu_sum, '(a, 2(1x,i5))') 
     :     ' nrec,nrec_max = ', nrec, nrec_max

      allocate( fas_out(imax,nrec), freq_out(imax,nrec) )

* Now read the data from the files:
     
      do i = 1, nrec

        allocate( ts(npts(i)), fas(npts(i)), freq(npts(i)) )

* Open time series file:

        write(*,'(1x,a)') 
     :   'Processing file:', f_smc(i)(1:nc_f_smc(i))

        call SMCRead(f_smc(i)(1:nc_f_smc(i)), tskip(i), tlength(i),
     :               ts,  
     :               char_head, int_head, real_head, comments)
        sps = real_head(2)
        npts_smc = int_head(17)
        dt = 1.0/sps
        t_smc(i) = float(npts_smc-1)*dt

        write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
     :     'signal_in: dt=', dt, 
     :     ' npts_smc=', npts_smc,' t_smc=', t_smc(i)

* Compute the Fourier spectra:

        call Abs_Spectra(ts, dt, npts_smc, 
     :    frctnfrtaper(i), fas, npw2, signnpw2(i), dc_remove(i), 
     :    df_fft)

        t_fft(i) = float(npw2-1)*dt
        
        write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)')
     :     'signal_fft: dt=', dt, ' npw2=', npw2,' t_fft=', t_fft(i)

* Smooth the spectra

        imode = 1        ! smooth amplitudes
        call SMTHS (fas, npw2, df_fft, df_smooth(i), imode)

* Set up the frequency array:

* First set up the frequency array for the spectra:

        nfreq = npw2/2 
        do j = 1, nfreq
          freq(j) = float(j-1)*df_fft
        end do

        if (df_intrp(i) .eq. 0.0) then
* use fft frequencies, between f_intrp_low and f_intrp_high
          call locate(freq, nfreq, f_intrp_low(i), m_start)
          m_start = m_start + 1
          call locate(freq, nfreq, f_intrp_high(i), m_stop)
          m_stop = m_stop - 1
          mspct = m_stop - m_start + 1
          do j = 1, mspct
            freq_out(j,i) = freq(j + m_start - 1)
          end do
        else
          mspct = (f_intrp_high(i)-f_intrp_low(i))/df_intrp(i) + 1
          do j = 1, mspct
            freq_out(j,i) = float(j-1) * df_intrp(i) + f_intrp_low(i)
          end do
        end if

        nfreq_out(i) = mspct

* Interpolate to specified frequencies:

        do j = 1, mspct
          fas_out(j,i) = yintrf(freq_out(j,i), freq, fas, nfreq)
        end do

        deallocate( ts, fas, freq )

      end do  ! end loop over computing spectra 

* Write output:

* Open output file:
      call trim_c(f_asc,nc_f_asc)
      call get_lun(nu_asc)
      open(unit=nu_asc,file=f_asc(1:nc_f_asc),status='unknown')
      write(*,'(1x,a)') 
     :   'Writing output to file:', f_asc(1:nc_f_asc)

      write(nu_sum,'(2a)') '   Output  file = ', f_asc

      write(nu_asc, '(a,1x,i2)') 
     :     ' Number of variables = ', 2*nrec
      write(nu_asc, '(a,1x,i3)') 
     :     ' Variable names start on line # ', 5+2*nrec+1
      write(nu_asc, '(a,1x,i3)') 
     :     ' Data starts on line # ', 5+2*nrec+2

      write(nu_asc, '(a)') 
     :    ' Output of program SMC2FAS:'
      write(nu_asc, '(2a)') ' File with input parameters: ', f_ctl
 
      call imnmax(nfreq_out,1,nrec,1,nfreq_min,nfreq_max) 
      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') 
     :     ' nfreq_min, nfreq_max = ', nfreq_min, nfreq_max

      do i = 1, nrec
        write(nu_asc, '(2a)') 
     :    ' File with time series: ',f_smc(i)
        write(nu_asc, '(2x, 2(1x,a, 1x,f8.2))') 
     :    ' t_smc= ',t_smc(i), ' t_fft=', t_fft(i)
      end do

      bigbuf = ' '
      iseg = 0
      do i = 1, nrec
        iseg = iseg + 1
        write(bigbuf((iseg-1)*21+1:(iseg-1)*21+21), 
     :    '(4x,a4,1x,a12)') 
     :    'freq', colhead(i)
      end do

      write(nu_asc,'(a)') bigbuf(1:len(bigbuf))

      write(*,'(a,1x,i5)') ' nfreq_max = ', nfreq_max
      do j = 1, nfreq_max   !  Account for unequal number of points:

        bigbuf = ' '
        iseg = 0
        do i = 1, nrec
          iseg = iseg + 1
          if (j .le. nfreq_out(i)) then      ! fill output string with value
            write(bigbuf((iseg-1)*21+1:(iseg-1)*21+21), 
     :      '(1x,f7.3,1x,1pe12.5)') 
     :       freq_out(j,i), fas_out(j,i)
          else
            write(bigbuf((iseg-1)*21+1:(iseg-1)*21+21), 
     :      '(7x,a1,12x,a1)') 
     :       '.', '.'
          end if
        end do ! loop over nrec
        write(nu_asc,'(a)') bigbuf(1:len(bigbuf))
      end do ! loop over nfreq


* Close files:

      close(nu_asc)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2FAS


      include '\forprogs\fork.for'
      include '\forprogs\tprfrctn.for'
      include '\forprogs\dcdt.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\smths.for'
      include '\forprogs\locate.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\zeropad.for'
      include '\forprogs\absspect.for'

