! ---------------------------------------------------------------- smc2rs
      Program SMC2RS

! Reads an SMC filename from a control file, computes PSV, and writes it to a
! column file

! !!!!!!!!!!!!!!!!!!!!!!!!
! Use if want output in column format;
! Use SMC2RS2 if want 91 periods in smc format
! !!!!!!!!!!!!!!!!!!!!!!!!

! The next line is the first line in a sample control file:
!!Control file for program SMC2RS
!!Use smc2rs2 for RS computed at a fixed set of periods up to 15 s
!! Revision of program involving a change in the control file on this date:
!   02/10/18
!!Summary file name:
! smc2rs.sum
!!Use PEER NGA periods? (y/n)
! n
!!Use tabulated periods? (y/n)
!! (Note: the response is ignored if the answer to the above is "y")
! y
!!If "y" above: Number of periods, followed by the periods:
!!If "n" above: Periods for response spectra (nperiods log spaced from 
!!              per_low to per_high):
!! Note: need two lines as placeholders if the the PEER NGA
!! periods are used 
! 6                         ! 200
! 5.0 2.0 1.0 0.5 0.2 0.1   ! 0.01 90.0
!!Damping values (ndamp, damp(1), damp(2), ..., damp(ndamp)):
! 3 0.02 0.05 0.20
!!t4mean (removes a mean from the whole record, where the mean is
!!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
!!    the mean of the whole record is used, and if t4mean < 0, no mean is
!!    removed. 
!! NOTE: need entries even if the parameter is not used (because read
!! info use list-directed [unformatted] input) 
!! Note that if working with a processed record that includes the 
!! complete time series produced by the processing (e.g., the padded time series
!! for acausal filtering or the post-processing baseline corrected time series
!! produced from pad-stripped or pass-through data in the first NGA project), 
!! it is inadvisible to remove the mean as this may 
!! produce unintended distortions at long periods; in this case set t4mean < 0.0.
!! If the record is pad-stripped, however, as provided by most
!! data centers, without post-processing to remove drifts in displacement,
!! a mean determined from the overall record probably should be 
!! removed.  But in this case the displacement will probably be inaccurate
!! (compared to the displacement derived from the padded, filtered acceleration
!! time series), it may be that a better approach is to specify the initial
!! velocity and displacement of the record---I am working on that (11/26/10):
! -10.0  
!!Sinc interpolate factor (IF) (this MUST be a power of two; e.g., 2, 4, 8, etc.;
!! IF = 1 returns the original data):
!! sps_out = interpolate_factor* sps_in
!! IF = 8 is high enough, because the sinc interpolation assumes that there
!! is no energy beyond the Nyquist data of the original data.  This is 0.5*sps_in.
!! If want to resolve an oscillator response with this frequency using at least 10 points
!! per period, this means resampling so that sps_out = 10 * 0.5*sps_in = 5 *sps_in.
!! Because a power of 2 is needed, IF = 8 is sufficient (see comparisons in 
!! Wang, P., J. P. Stewart, Y. Bozorgnia, D. M. Boore, and T. Kishida (2017).
!! "R" Package for Computation of Earthquake Ground-Motion Response Spectra,
!! PEER 2017/09, Pacific Earthquake Engineering Research Center, University of California, Berkeley, California.
! 8
!!Use straightline interpolation if Tosc < 10*dt (this interpolates the acceleration time series to finer spacing before
!!computing the response spectra; it was used before the incorporation of sinc interpolation, and is generally not needed
!!when sinc interpolation is done)
! Y
!!character string to append to filename to make the output filename:
! smc2rs.col
!!tskip tlength  
! 0.0 400.0
!!Are the units of acceleration in g (Y/N); if N, I assume that the units are
!! cm/s/s.  If something else, it is up to the user to scale the output appropriately 
!! (the output units are as follows: sd(cm); pv(cm/s); pa(cm/s/s); rv(cm/s); aa(cm/s/s); ag (g)
!! where the output columns indicate the type of response spectra, as follows:
!! sd=relative displacement
!! pv=pseudo relative velocity
!! pa=pseudo absolute acceleration (aka PSA)
!! rv=relative velocity
!! aa=absolute acceleration
!! ag=pseudo absolute acceleration, in g.
!! (see aa_pa_rv_pv_2.pdf on the Dave's informal notes on a variety of topics page of www.daveboore.com)
! N
!!accel_file names 
!!("STOP" in any column, any case, to quit)
!ARL090.AT2.smc8 
!ARL360.AT2.smc8
!TAB-LN.AT2.smc8
!TAB-TR.AT2.smc8
!ARL090.AT2.smc 
!ARL360.AT2.smc 
!TAB-LN.AT2.smc 
!TAB-TR.AT2.smc 
! StoP
  
! Dates: 03/05/10 - Written by D. Boore, based on SMC2RS, using RSP_CMPMAX_UCMPMX
!                   instead of RDRVAA. RSP_CMPMAX_UCMPMX interpolates the
!                   acceleration time series to smaller dt if period < 10.0*dt
!                 - This version calls RSP_RDRVAA_UCMPMX, which will use 
!                   rdrvaa for the bulk of the calculations (it seems to be
!                   faster than CMPMAX).
!        03/06/10 - Substituted RSCALC_INTERP_ACC for RSP_RDRVAA_UCMPMX
!                 - The new response spectral computation subroutine seems to work,
!                   so I'll convert SMC2RS_RSCALC.FOR to SMC2RS.FOR (the main routine
!                   for computing response spectra from smc files).
!                 - Deleted option to use d0, v0
!        04/04/10 - Removed conversion of f_smc to upper case; introduce buf4
!                   as a substitute to check for 'STOP' in the control file.
!        06/30/10 - Correct error when ndamp > 1
!        06/30/13 - Use Fortran 95 extension getcl to get a command line argument that determines if
!                   the default control file name is used (this will be used for calling the main
!                   program within a batch file (c:\forprogs\smc2rs -default_ctl)
!        11/07/16 - Do not include the path of the input file in the output file name.
!        02/09/18 - A major revision, allowing sinc interpolation in the program (based on 
!                   smc_interpolate_time_series_using_fork.for)
!                 - Delete stuff related to xfactr (see comments below from a previous version about why
!                   this was not a useful processing parameter).
!!Xfactr, used to lengthen time series to max(tend_in, xfactr * per_high)
!! Note that this is a relic from a time when I thought that this would be
!! an important parameter.  In fact, adding zeros to the input acceleration
!! trace usually leads to an effective step in baseline and consequently
!! a disastrous increase in long period response.   In almost all cases
!! xfactr should be set to 0.0.  Because the oscillator response for a long
!! period is close to the ground displacement, extending the time series
!! is not needed unless the displacement has unbounded growth, in which case
!! the long-period oscillator response is meaningless
!                 - Ask if units of accel are g.
!        02/27/18 - Correct error in assigning skip_straightline_interp (it
!                   was set to T when the input from the control file asking
!                   if linear interpolation was to be used was set to Y). The logical 
!                   way to do this is to change skip_straightline_interp to use_straightline_interp.
!                   This requires changes to a number of routines that use rs_calc, but
!                   is the right thing to do.

      implicit none

      character date_ctl_correct*8, date_ctl_in*30

      real(8) :: cum_avg

      real(4) :: twopi, per_high, per_low, dlogt, t4mean_in, tlength, 
     :           tskip, tend_in, rnull, v0, d0, t4mean, avg, 
     :           amax, amin, tmax, g2gals 

      integer :: nc_date_ctl_in, nc_date_ctl_correct, 
     :           nc_cl_arg, nc_f_ctl, nu_ctl, 
     :           nc_cmnts2skip, nc_f_sum, nu_sum,
     :           nc_buf_c, i, nperiods, ndamp, nc_s2appnd, nc_f_smc,
     :           ncomments, n_t4mean, indx_max, indx_min, npts,
     :           imax, nc_path, nu_rs, nc_f_rs, i_stem, iticks1,
     :           iper, idamp, iticks2, nper_nga
     
      
      character char_head(11)*80, comments(200)*80, buf_out*80, buf4*4
      character cmnts2skip(20)*80, string_to_append*40
      integer int_head(48), interpolate_factor,
     : npts_in, npts_out
      
      real real_head(50), sps_in, sps_out, dt_in, dt_out
      real a_in(:) , a_out(:)
      allocatable :: a_in, a_out 
      character f_sum*300
      character f_smc*300, f_rs*300, buf_c*300, path*300
      real rd(5), rv(5), aa(5), damp(5), period(1000)
      real omega, omega2
      character header(33)*12, damp_c*2, cl_arg*50

      character f_ctl*120
      logical f_exist, use_v0_d0, use_nga_per, use_tabulated_per,
     :        use_straightline_interp, accel_units_g

      real per_nga(105) ! does NOT include pgv and pga as per = -1.0, 0.0
      data per_nga/
     :   0.010,0.020,0.022,0.025,0.029,0.030,0.032,0.035,0.036,0.040,
     :   0.042,0.044,0.045,0.046,0.048,0.050,0.055,0.060,0.065,0.067,
     :   0.070,0.075,0.080,0.085,0.090,0.095,0.100,0.110,0.120,0.130,
     :   0.133,0.140,0.150,0.160,0.170,0.180,0.190,0.200,0.220,0.240,
     :   0.250,0.260,0.280,0.290,0.300,0.320,0.340,0.350,0.360,0.380,
     :   0.400,0.420,0.440,0.450,0.460,0.480,0.500,0.550,0.600,0.650,
     :   0.667,0.700,0.750,0.800,0.850,0.900,0.950,1.000,1.100,1.200,
     :   1.300,1.400,1.500,1.600,1.700,1.800,1.900,2.000,2.200,2.400,
     :   2.500,2.600,2.800,3.000,3.200,3.400,3.500,3.600,3.800,4.000,
     :   4.200,4.400,4.600,4.800,5.000,5.500,6.000,6.500,7.000,7.500,
     :   8.000,8.500,9.000,9.500,10.000/
      nper_nga = 105      

      twopi = 2.0*4.0*atan(1.)

      f_exist = .false.
      cl_arg = ' '
      call getcl(cl_arg)
      call trim_c(cl_arg, nc_cl_arg)
      if (nc_cl_arg > 0) then
        f_ctl = ' '
        f_ctl = 'smc2rs.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
          if (.not. f_exist) then
            write(*,'(a)') 
     :             ' ******* CONTROL FILE '//f_ctl(1:nc_f_ctl)//
     :             ' DOES NOT EXIST; QUITTING! ******* '
            stop
          end if
      else 
        do while (.not. f_exist)
          f_ctl = ' '
          write(*, '(a\)') 
     :      ' Enter name of control file '//
     :      '(Enter=smc2rs.ctl;'//
     :      ' ctl-brk to quit): '
          read(*, '(a)') f_ctl
          if (f_ctl(1:4) == '    ') 
     :      f_ctl = 'smc2rs.ctl'
          call trim_c(f_ctl, nc_f_ctl)
          inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
          if (.not. f_exist) then
            write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
          end if
        end do
      end if

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

! Check date of control file:
      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      date_ctl_correct = ' '
      date_ctl_correct = '02/10/18'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
! Check date of control file:

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum, nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Summary file for program SMC2RS'
      write(nu_sum,'(2a)') '   Using control file: ', f_ctl

! Read period info and fill period array:
        
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call upstr(buf_c)
      call trim_c(buf_c,nc_buf_c)
      if (buf_c(1:1) == 'Y') then
        use_nga_per = .true.
      else
        use_nga_per = .false.
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call upstr(buf_c)
      call trim_c(buf_c,nc_buf_c)
      if (buf_c(1:1) == 'Y') then
        use_tabulated_per = .true.
      else
        use_tabulated_per = .false.
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      if (use_nga_per) then  ! skip the next two lines
        call skip(nu_ctl,2)
        nperiods = nper_nga
        do i = 1, nperiods
          period(i) = per_nga(i)
        end do
        write(nu_sum,'(a)') ' Use PEER NGA periods'
      else if (use_tabulated_per) then  ! read info for periods and fill array
        read(nu_ctl,*) nperiods    
        read(nu_ctl,*) (period(i), i=1, nperiods)      
        write(nu_sum,'(a)') ' Use PEER NGA periods: '
        do i = 1, nperiods
          write(nu_sum,*) period(i)
        end do
      else  ! not nga, not tabulated
        read(nu_ctl,*) nperiods 
        read(nu_ctl,*) per_low, per_high
        dlogt = alog10(per_high/per_low)/float(nperiods-1)
        do i = 1, nperiods
          period(i) = per_low*10.0**(float(i-1) * dlogt)
        end do
        write(nu_sum,'(a)') 
     :       ' Construct periods, using nperiods, per_low, per_high= '
        write(nu_sum,*) nperiods, per_low, per_high
      end if

        
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) ndamp, (damp(i), i = 1, ndamp)
      write(nu_sum,*) ndamp, (damp(i), i = 1, ndamp)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   interpolate_factor  ! This must be a power of two--check
      if (modulo(log(real(interpolate_factor)), log(2.0)) /= 0) then
        print *,' interpolate_factor= ', 
     :              interpolate_factor, ' not a power of 2; QUIT!!'
        close(nu_ctl)
        stop
      end if

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call upstr(buf_c)
      call trim_c(buf_c,nc_buf_c)
      if (buf_c(1:1) == 'Y') then   
        use_straightline_interp = .true.
      else
        use_straightline_interp = .false.
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      string_to_append = ' '
      read(nu_ctl,'(a)') string_to_append
      call trim_c(string_to_append, nc_s2appnd)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) tskip, tlength      
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call upstr(buf_c)
      call trim_c(buf_c,nc_buf_c)
      if (buf_c(1:1) == 'Y') then
        accel_units_g = .true.
      else
        accel_units_g = .false.
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      loop_over_smc_files : DO

        f_smc = ' '
        read(nu_ctl,'(a)') f_smc
        call trim_c(f_smc,nc_f_smc)
        buf4 = ' '
        buf4 = f_smc(1:4)
        call upstr(buf4)
        if(buf4(1:4) == 'STOP') EXIT

        f_exist = .false.
        inquire(file=f_smc(1:nc_f_smc), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE
        else
          write(*,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)
          write(nu_sum,*)
          write(nu_sum,'(a)') ' f_smc = '//f_smc(1:nc_f_smc)
        end if
 
        write(nu_sum,*)

! Get npts and allocate the array:

        call smc_npts(f_smc(1:nc_f_smc), npts_in, sps_in)
        write(*,'(a,1x,i6)') ' After smc_npts, npts_in = ', npts_in
      
        allocate (a_in(npts_in))

! Read the input file:

        do i = 1, npts_in
          a_in(i) = 0.0
        end do
        
        call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :               a_in, char_head, int_head, real_head, comments)
       
        sps_in = real_head(2)
        dt_in = 1.0/sps_in
        npts_in = int_head(17)
        tend_in = float(npts_in-1)/sps_in
        ncomments = int_head(16)
  
! Scale to cm/s/s if in g:
        if (accel_units_g) then
          g2gals = 981.0
          do i = 1, npts_in
            a_in(i) = g2gals * a_in(i)
          end do
        end if
        

        rnull = real_head(1)
        v0 = 0.0
        d0 = 0.0
        use_v0_d0 = .false.
        if (use_v0_d0) then
          v0 = real_head(38)
          if (abs(v0) == abs(rnull)) then
            v0 = 0.0
          end if
          d0 = real_head(39)
          if (abs(d0) == abs(rnull)) then
            d0 = 0.0
          end if
        end if

        write(nu_sum,'(a)') ' sps_in, ncomments, npts_in = '
        write(nu_sum, *) sps_in, ncomments, npts_in

! Step 1: Remove a mean from the whole record

        t4mean = t4mean_in

!  Determine the appropriate mean:
        if(t4mean < 0.0) then  ! do not remove a mean
          avg = 0.0
        else
          if (t4mean == 0.0 .or. t4mean > tend_in) then   
            t4mean = tend_in  ! use whole record
          end if
          n_t4mean = sps_in * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i))
          end do
          avg = cum_avg/float(n_t4mean)
!          avg = sngl(cum_avg/n_t4mean)
!  Remove the mean from the whole record:
          do i = 1, npts_in
            a_in(i) = a_in(i) - avg
          end do
        end if

! Step 2: Derive a new time series using sinc interpolation:

        sps_out =  real(interpolate_factor) * sps_in
        npts_out = interpolate_factor * npts_in

        allocate( a_out(npts_out) )

        write(nu_sum,'(a)') ' Before call to interpolate, '//
     :     'sps_in, sps_out, npts_in, npts_out = '
        write(nu_sum, *) sps_in, sps_out, npts_in, npts_out

        call interpolate_time_series_using_fork(
     :   a_in, sps_in, npts_in, 
     :   interpolate_factor,
     :   a_out, sps_out, npts_out
     :   )

        dt_out = 1.0/sps_out
        
        write(nu_sum,'(a)') 
     :     ' interpolate_factor, sps_out, dt_out, npts_out = '
        write(nu_sum, *) 
     :       interpolate_factor, sps_out, dt_out, npts_out

        deallocate(a_in)

! Step 3: compute response spectra from the sinc interpolated time series:


! Find time of acceleration maximum:
        call mnmaxidx( a_out,1,npts_out,1,
     :                 amin,amax,indx_min,indx_max) 
        imax = indx_min
        if (abs(amax) > abs(amin)) then
          imax = indx_max
        end if
        tmax = float(imax - 1)*dt_out
        write(nu_sum,'(a)') 
     :     ' amin,amax,indx_min,indx_max,imax,tmax = '
        write(nu_sum,'(1p2(1x,e10.3),0p3(1x,i6),1p1x,e10.3)') 
     :       amin,amax,indx_min,indx_max,imax,tmax

! Open output file, set up and write column labels

        path = ' '
        call get_path_from_file_name(f_smc,nc_f_smc,path,nc_path)

        call get_lun(nu_rs)
        f_rs = ' '
        f_rs = f_smc(nc_path+1:nc_f_smc)//string_to_append(1:nc_s2appnd)
        call trim_c(f_rs,nc_f_rs)
       
        write(*,'(a)') ' f_rs = '//f_rs(1:nc_f_rs)

        open(unit=nu_rs,file=f_rs(1:nc_f_rs),status='unknown')

        write(nu_rs,'(1x,a, 1x,i2, 1x,a, 1x,l1)') 
     :     '! interpolate_factor = ', interpolate_factor,
     :     '; use_straightline_interp = ', use_straightline_interp

        i_stem = index(f_rs(1:nc_f_rs),'.')
        header(1) = 'per:________'
        header(1)(5:i_stem+3) = f_rs(1:i_stem-1) 
        header(2) = 'freq:_______'
        header(2)(5:i_stem+3) = f_rs(1:i_stem-1) 
        do i = 1, ndamp
          write(damp_c,'(i2.2)') nint(100.0*damp(i))
          header(6*(i-1)+3) = 'sd'//damp_c//'________'
          header(6*(i-1)+3)(5:i_stem+3) = f_rs(1:i_stem-1) 
          header(6*(i-1)+4) = 'pv'//damp_c//'________'
          header(6*(i-1)+4)(5:i_stem+3) = f_rs(1:i_stem-1) 
          header(6*(i-1)+5) = 'pa'//damp_c//'________'
          header(6*(i-1)+5)(5:i_stem+3) = f_rs(1:i_stem-1) 
          header(6*(i-1)+6) = 'rv'//damp_c//'________'
          header(6*(i-1)+6)(5:i_stem+3) = f_rs(1:i_stem-1) 
          header(6*(i-1)+7) = 'aa'//damp_c//'________'
          header(6*(i-1)+7)(5:i_stem+3) = f_rs(1:i_stem-1) 
          header(6*(i-1)+8) = 'ag'//damp_c//'________'
          header(6*(i-1)+8)(5:i_stem+3) = f_rs(1:i_stem-1) 
        end do
  
        write(nu_rs,'(33(1x,a12))') (header(i),i=1,6*ndamp+2)

! Loop over periods and damping, computing and writing response spectra:

        call timer(iticks1)
        do iper = 1, nperiods
          omega = twopi/period(iper)
          omega2 = omega**2
          buf_out = ' '
          buf_out = '+ Patience! Computing rd for period '
          write(buf_out(37:44),'(f8.3)') period(iper) 
          write(*,'(a)') buf_out
          do idamp = 1, ndamp
            call rs_calc(a_out, npts_out, omega, 
     :                   damp(idamp), dt_out, 
     :                   v0, d0, use_straightline_interp, 
     :                   rd(idamp), rv(idamp), aa(idamp) )
     
          end do ! loop over damping
          write(nu_rs,'(1p33(1x,e12.5))')
     :     period(iper), 1.0/period(iper),
     :     (rd(i), omega*rd(i), omega2*rd(i), rv(i), aa(i), 
     :      omega2*rd(i)/981.0,i = 1, ndamp) 
        end do ! loop over nperiods
        close(nu_rs)

        call timer(iticks2)
      
        write(nu_sum,'(1x,a, 1x,f6.2)') 
     :   ' compute time, in secs, for rs calculation = ', 
     :                 float(iticks2-iticks1)/100.0

        deallocate (a_out)

      END DO loop_over_smc_files
      
      close(nu_ctl)
      close(nu_sum)
 
      end
! ---------------------------------------------------------------- smc2rs

      include '\forprogs\rs_calc.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\get_npw2.for'      
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_path_from_file_name.for'
      include '\forprogs\interpolate_time_series_using_fork.for'
      include '\forprogs\fork.for'

 