! ------------------------------------------------------------------- SMC2Spectral_Moments
      Program SMC2Spectral_Moments


! Computes the spectral moments used in random vibration theory
! from the Fourier amplitude spectra of the specified time series.  
 
* The output file name has the same name as the input file, with the specified
* character string appended to the name.

* Input from a control file; here is an example:
*!Control file for program SMC2Spectral_Moments
*! Revision of program involving a change in the control file on this date:
*   02/06/11
*! As many comment lines as desired, each starting with "!"
*! The string "pp:" indicates a new set of processing parameters
*! to be applied to the following smc files.  The parameters are given on the
*! lines following "pp:", until the next "pp:" line or until "stop" is 
*! encountered.
*! NOTE: Use the tapers with caution, choosing them so that important signal
*! is not reduced by the tapering.  This can be particularly a problem with 
*! analog data from relatively small earthquakes that triggered near the 
*! S-wave arrival.  
*!
*!Name of summary file:
* smc2spectral_moments.sum
*PP: new set of parameters
*!tskip, tlength
*   0.0 2000.0
*!dc_remove?
*  .false.        
*!Length of taper at beginning and end of time series, before adding zeros
*! to make the number of points in the record a power of two.
* 0.0 0.0
*!extend normal padded length by an additional factor of 2**npower (npower = 0 => no extension
*! npower = 2 => extend by a factor of four, etc.):
*  0
*!character string to append to filename:
*   .spectral_moments_no_extension_t2.0.col
*!Tosc
*  2.0
*!Files to process:
* A11.V01.a5
*PP: new set of parameters
*!tskip, tlength
*   0.0 2000.0
*!dc_remove?
*  .false.        
*!Length of taper at beginning and end of time series, before adding zeros
*! to make the number of points in the record a power of two.
* 0.0 0.0
*!extend normal padded length by an additional factor of 2**npower (npower = 0 => no extension
*! npower = 2 => extend by a factor of four, etc.):
*  2
*!character string to append to filename:
*   .4x_extension_t2.0
*!Tosc
*  2.0
*!Files to process:
* A11.V01.a5
*stop
 
* Dates: 02/06/11 - Written by D.M. Boore, patterned after SMC2FS2
c
c Dimension and declaration statements:

      real fas(:), freq_fft(:), ts(:), yint(:)
      allocatable :: fas, freq_fft, ts, yint
      
      real amom_integrand(:,:)
      allocatable amom_integrand

      integer npts, nfreq_out, nc_f_smc
      real tskip, tlength, ftaper
      real signnpw2
      real t_smc, t_fft
      real amom(5)

      character char_head(11)*80, comments(500)*80, cmnts2skip(500)*80
      real real_head(50)
      integer int_head(48)

      character buf*80, buf_upper*4, 
     :        f_smc*150, 
     :        f_out*150, f_sum*80, c2*2, c2up*2,
     :        ext_c*10, string*80, string4name*80,
     :        specify_frequencies*10, f_integrand*150
     
      logical reset_smc_out

      character f_ctl*60, date*8, 
     :        time_begin*10, time_start*10, time_stop*10 

      character date_ctl_correct*8, date_ctl_in*30
      
      logical f_ctl_exist, f_smc_exist, dc_remove, datetime_l
      
      twopi = 2.0*4.0*atan(1.0)

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :    '("enter"=smc2spectral_moments.ctl;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smc2spectral_moments.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '02/06/11'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2Spectral_Moments:'
      date = ' '
      time_begin = ' '
* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time_begin )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'
      write(nu_sum, '(2a)') ' Date: '//date(1:4)//'/'//
     :                                 date(5:6)//'/'//date(7:8)
      write(nu_sum, '(2a)') ' Time: '//time_begin(1:2)//':'//
     :           time_begin(3:4)//':'//time_begin(5:10)
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)

      loop_over_files: DO
        buf = ' '
        read(nu_ctl,'(a)',IOSTAT=iostatus) buf
        if (iostatus /= 0) EXIT
        call trim_c(buf,nc_buf)
        buf_upper = ' '
        buf_upper = buf(1:4)
        call upstr(buf_upper)
        if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') CYCLE
        if(buf_upper(1:4) .eq. 'STOP') EXIT
      
        processing_parameters: IF(buf_upper(1:3) .eq. 'PP:') THEN  ! a new set of processing parameters

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) tskip, tlength_in
          write(nu_sum,'(a)') 
     :     ' tskip, tlength_in ='
          write(nu_sum,*) 
     :       tskip, tlength_in
     
          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) dc_remove
          write(nu_sum,'(a,1x,l1)')  ' dc_remove = ', dc_remove

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) taper_front, taper_back
          write(nu_sum,'(a)') 
     :     ' taper_front, taper_back ='
          write(nu_sum,*) 
     :       taper_front, taper_back

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) npower
          write(nu_sum,'(a)') 
     :     ' npower ='
          write(nu_sum,*) 
     :       npower

          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          string4name = ' '
          read(nu_ctl,'(a)') string4name
          call trim_c(string4name, nc_string4name)
          write(nu_sum,'(a)') 
     :   ' string4name = '//string4name(1:nc_string4name)


          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
          read(nu_ctl,*) tosc
          write(nu_sum,'(a)') 
     :     ' tosc ='
          write(nu_sum,*) 
     :       tosc
 
          call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

          CYCLE
          
        END IF processing_parameters 

      
        f_smc = ' '
        f_smc = buf(1:nc_buf)
        call trim_c(f_smc, nc_f_smc)
        inquire(file=f_smc(1:nc_f_smc), exist=f_smc_exist)
        if (.not. f_smc_exist) then
            write(*,'(a)') ' ****** FILE '//f_smc(1:nc_f_smc)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE
        end if

* Passed tests; proceed

* Extract the time series from the input file:

        print *
        print *, 
     :   'Processing file:', f_smc(1:nc_f_smc)
        write(nu_sum,*)
        write(nu_sum,'(1x,a)') 
     :   'Processing file:', f_smc(1:nc_f_smc)

        time_start = ' '
* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
        call DATE_AND_TIME( date, time_start )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'
        write(nu_sum, '(2a)') '  Time start processing record: '//
     :           time_start(1:2)//':'//
     :           time_start(3:4)//':'//time_start(5:10)
 
* Get number of points on this loop:

      call smc_npts(f_smc(1:nc_f_smc), npts, sps)

      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') ' npts = ', npts

! Allocate time series:

      signnpw2 = +1.0
      write(nu_sum,*)
      write(nu_sum, '(a, 1x,i5, 1x,f4.1)') 
     :    ' before call to get_npw2: npts, signnpw2 = ', 
     :      npts, signnpw2 
      call get_npw2(npts,signnpw2,npw2)
      write(nu_sum,*)
      write(nu_sum, '(a, 1x,i5, 1x,f4.1, 1x,i5)') 
     :    ' after call to get_npw2: npts, signnpw2, npw2 = ', 
     :      npts, signnpw2, npw2
      npw2 = npw2*2**npower

      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') ' npower, npw2 = ', npower, npw2

      allocate( ts(npw2), fas(npw2), freq_fft(npw2) )  
! In order to force abs_spectra to add more than the usual number of 
! zeros, need to allocate ts with npw2 here rather than npts_smc, 
! and reset npts_smc to npw2

! initialize arrays
      do i = 1, npw2
        ts(i) = 0.0
        fas(i) = 0.0
        freq_fft(i) = 0.0
      end do

* Now read the data from the files:

      tlength = tlength_in
      call SMCRead(f_smc(1:nc_f_smc), tskip, tlength,
     :               ts,  
     :               char_head, int_head, real_head, comments)
      sps = real_head(2)
      npts_smc_in = int_head(17)
      dt = 1.0/sps
      t_smc_in = float(npts_smc_in)*dt
      
      nacc_start = tskip * sps
      nacc_stop  = nacc_start + tlength * sps

      write(nu_sum,'(3x,a,f8.6,a,i6,a,f8.2)') 
     :     'signal_in: dt=', dt, 
     :     ' npts_smc_in =', npts_smc_in,' t_smc_in=', t_smc_in
      write(nu_sum,'(3x,a,1x,f8.2)') 
     :     ' tlength after smcread = ', tlength
      write(nu_sum,'(3x,a,i6)') 
     :     ' npw2 =', npw2 

* Compute the Fourier spectra:

      npts_smc = npw2
! In order to force abs_spectra to add more than the usual number of 
! zeros, need to allocate ts with npw2 here rather than npts_smc, 
! and reset npts_smc to npw2

      CALL Abs_Spectra(ts, dt, npts_smc, dc_remove, 
     :  taper_front, taper_back, fas, npw2, signnpw2, 
     :  df_fft, nzpad)

      t_fft = float(npw2)*dt
        
      nfreq_fft = npw2/2 

* Set up the frequency array for output:

* First set up the frequency array for the spectra:

      DO j = 1, nfreq_fft
        freq_fft(j) = float(j-1)*df_fft
      END DO
 
      write(nu_sum,*) ' signnpw2, nzpad, npw2, t4fft = ',
     :      signnpw2, nzpad, npw2, real(npw2)*dt     
 
      deallocate( ts )

! Compute spectral moments:

      damp = 0.05
      idva = 2
      fosc = 1.0/tosc

      allocate (amom_integrand(nfreq_fft, 5))
      
      DO i = 1, 5
        DO j = 1, nfreq_fft
          f = freq_fft(j)
          amom_integrand(j,i) = 
     :        f**(i-1)*( harmoscf(f, fosc, damp, idva)*fas(j) )**2
        END DO
        allocate (yint(nfreq_fft))
        yint_0 = 0.0
        call trapezoidal_integration(amom_integrand(1,i), nfreq_fft, 
     :                               df_fft, yint_0, yint)
        amom(i) = 2.0*twopi**(i-1)*yint(nfreq_fft)
        deallocate(yint)
      END DO
      
* Write output:

      f_integrand = ' '
      f_integrand = f_smc(1:nc_f_smc)//
     :        string4name(1:nc_string4name)//'.integrands.out'
      call trim_c(f_integrand,nc_f_integrand)
      call get_lun(nu_integrand)
      open(nu_integrand,f_integrand(1:nc_f_integrand),status='unknown')
      write (nu_integrand,'(
     :         9x,a, 
     :         5(7x,a))
     :                       ') 
     :   'freq', 
     :   'mom(0)', 'mom(1)','mom(2)','mom(3)','mom(4)'
      
      do j = 1, nfreq_fft
        write(nu_integrand, '(6(1x,es12.5))')
     :    freq_fft(j), (amom_integrand(j,i), i = 1, 5)
      end do
      close(nu_integrand)      
      deallocate(amom_integrand)
          
      deallocate (fas, freq_fft)        

* Construct output filename:
  
      f_out = ' '
      f_out = f_smc(1:nc_f_smc)//
     :        string4name(1:nc_string4name)//'.moments.out'
      call trim_c(f_out,nc_f_out)

      write(*,*) 
     :   'Writing output to file:', f_out(1:nc_f_out)
      write(nu_sum,'(2a)') ' Output file = ', f_out(1:nc_f_out)

      call get_lun(nu_out)
      open(nu_out,f_out(1:nc_f_out),status='unknown')
      
      write (nu_out,'(
     :         5(7x,a))
     :                       ') 
     :   'mom(0)', 'mom(1)','mom(2)','mom(3)','mom(4)'
     
      write(nu_out,'(5(1x,es12.5))') 
     :    (amom(i), i = 1, 5)
     
      close(nu_out)
      
      
      string = ' '
      string = '| Output of program SMC2FS2:'
      call trim_c(string, nc_string)
 
      datetime_l = .true.

 
      time_stop = ' '
* Standard Fortran 90 intrinsic Subroutine DATE_AND_TIME
      call DATE_AND_TIME( date, time_stop )
* Date is returned as 'CCYYMMDD'
* Time is returned as 'hhmmss.sss'
      write(nu_sum, '(2a)') '  Time finished processing record: '//
     :           time_stop(1:2)//':'//
     :           time_stop(3:4)//':'//time_stop(5:10)

      call time_diff(time_start, time_stop, time_elapsed)
      write(nu_sum, '(2x,a,1x,1p, e10.3)') 
     :       ' Elapsed time (sec): ', time_elapsed
      write(nu_sum,*)
      write(nu_sum,*)

      END DO loop_over_files

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMC2FS2


      include '\forprogs\abs_spectra.for'
      include '\forprogs\harmoscf.for'
      include '\forprogs\trapezoidal_integration.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\locate.for'
      include '\forprogs\yintrf.for'
      include '\forprogs\fs2write.for'
      include '\forprogs\get_date.for'
      include '\forprogs\get_time.for'
      include '\forprogs\time_diff.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\fork.for'
      include '\forprogs\tprfrctn.for'
      include '\forprogs\dcdt.for'
      include '\forprogs\zeropad.for'
 