* ---------------------------------------------------------------- SMC2Subd
      Program SMC2Subd

* Reads an SMC filename from a control file, computes PSV, and writes it to a
* file in the same format as Gail Atkinson's SUBALL.DAT.

* Damping and oscillator frequencies are hardwired into the program

* The next line is the first line in a sample control file:
*Control file for program SMC2SUBD
*Name of Summary File:
* SMC2SUBD.SUM
*Name of Output File:
* NI01_CR.DAT
*Comments for line 1 of output:
* Western Washington, 1949:
*Identifier for first field of output (up to 10 characters):
*NI010228
*Eq Name:
* Nisqually, WA
*Eq Hypocenter:
* 47.1525, -122.7197
*Depth:
* 52.40
*Moment Magnitude:
* 6.8
*Type ("sub", "sla")
* sla
*Station Filenames:
*ALK0_A.SMC
*ALK1_A.SMC
*ALO1_A.SMC
*ALO2_A.SMC
*BHD1_A.SMC
*BHD2_A.SMC
*BOE1_A.SMC
*BOE2_A.SMC
*Stop


* Dates: 06/05/01 - Written by D. Boore, based on SMC2RS and UW2GS
*        06/06/01 - Use fosc = 0.3333 rather than 0.33
*        06/13/01 - Read string for first field of output (eq_id) from ctl file
*                   rather than its being hardwired
*        07/18/01 - New distance (RCD2F), better output
*          07/17/03 - Add sps parameter to smc_npts
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
 
      character char_head(11)*80, comments(200)*80, ctype*3, c_l1*80
      integer int_head(48)
      
      real real_head(50)
      real a(:)
      allocatable :: a
      character f_sum*40, eq_id*10
      character f_smc*40, f_rs*40, buf_c*79
      real rd(7), rv(7), aa(7)
      real omega, omega2(7)
      character orient_c*5, eq_name*80

      character f_ctl*40
      logical f_ctl_exist

      real fosc(7)
      data fosc/ 0.3333, 0.50, 1.0, 2.5, 5.0, 10.0, 25.0/
      nfosc = 7

      twopi = 2.0*4.0*atan(1.)

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') stop
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl,status='unknown')

      call skip(nu_ctl,2)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum,status='unknown')
      write(nu_sum,'(a)') ' Output of program SMC2Subd'

      call skip(nu_ctl,1)
      f_rs = ' '
      read(nu_ctl,'(a)') f_rs
      call get_lun(nu_subnew)
      open(unit=nu_subnew,file=f_rs,status='unknown')
     
      call skip(nu_ctl,1)
      c_l1 = ' '
      read(nu_ctl,'(a)') c_l1
      write(nu_subnew,'(a)') c_l1

      write(nu_subnew,'(
     :   7x,a, 4x,a, 3x,a, 5x,a, 1x,a, 4x,a, 5x,a, 
     :   7x,a, 6x,a, 7x,a, 7x,a, 7x,a, 7x,a, 7x,a, 7x,a,
     :   4x,a, 5x,a, 3x,a, 1x,a)')  
     :  'date', 'h', 'M', 'stn', 'class', 'vs', 'R',
     :  'PGA', '0.33', '0.5', '1.0', '2.5', '5.0', '10.', '25.',
     :  'lat', 'lon', 'M', 'typ'

      call skip(nu_ctl,1)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call trim_c(buf_c,nc_buf_c)
      eq_id = ' '
      eq_id = buf_c(1:10)

      call skip(nu_ctl,1)
      eq_name = ' '
      read(nu_ctl,'(a)') eq_name
      call trim_c(eq_name,nc_eq_name)

      call skip(nu_ctl,1)
      read(nu_ctl, *) eqlat, eqlong

      call skip(nu_ctl,1)
      read(nu_ctl, *) eqdepth

      call skip(nu_ctl,1)
      read(nu_ctl, *) eqmag

      call skip(nu_ctl,1)
      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call trim_c(buf_c,nc_buf_c)
      ctype = ' '
      ctype = buf_c(1:3)
      
      call skip(nu_ctl,1)
     
1000  continue

      buf_c = ' '
      read(nu_ctl,'(a)') buf_c
      call upstr(buf_c)
      call trim_c(buf_c,nc_buf)
      if(buf_c(1:4) .eq. 'STOP') then
         close(nu_ctl)
         close(nu_sum)
         close(nu_subnew)
         stop
      end if

      f_smc = ' '
      f_smc = buf_c(1:nc_buf)
      call trim_c(f_smc,nc_f_smc)      

      write(*,'(a)') 
     :  ' Working on file '//f_smc(1:nc_f_smc)

      write(nu_sum,'(a)') 
     :  ' Working on file '//f_smc(1:nc_f_smc)

* Get npts and allocate the array:

      call smc_npts(f_smc, npts, sps)

      allocate (a(npts))

* Read the input file:

      do i = 1, npts
        a(i) = 0.0
      end do

      tskip = 0
      tlength = 1000.0
      call SMCRead(f_smc, tskip, tlength,
     :               a, char_head, int_head, real_head, comments)

      orient_c = ' '
      orient_c = char_head(6)(53:57)

      sps = real_head(2)
      dt = 1.0/sps
      ncomments = int_head(16)

      rnull = real_head(1)
      v0 = real_head(38)
      if (abs(v0) .eq. abs(rnull)) then
        v0 = 0.0
      end if
      d0 = real_head(39)
      if (abs(d0) .eq. abs(rnull)) then
        d0 = 0.0
      end if

      write(*,'(a,1x,f3.1,1x,f3.1)') ' d0, v0 = ', d0, v0


      npts = int_head(17)

      stalat = real_head(11)
      stalong = real_head(12)

* Compute epicentral and hypocentral distances:
      call distaz( -1.0, eqlat,eqlong,stalat,stalong,
     :    rdeg, rep, az, baz)

* Do calcs for dist and set mag:
      flength = 10.0**(-2.44+0.59 * eqmag)
      farea   = 10.0**(-3.49+0.91 * eqmag)
      fwidth  = farea/flength
      depthp = eqdepth - 0.35 * fwidth
      repp = rep - 0.32*flength
      rcd2f = sqrt(repp**2 + depthp**2)

      write(*,'(a)') ' rep, flength, repp, eqdepth, depthp, rcd2f = '
      write(*,*)       rep, flength, repp, eqdepth, depthp, rcd2f

      write(nu_sum,'(a)') ' sps, ncomments, npts = '
      write(nu_sum, *) sps, ncomments, npts

      write(nu_sum,'(a)') ' orient_c = '//orient_c
      write(nu_sum,'(a,1x,i6,1x,i6)') 
     :   ' ivorient, ihorient = ',
     :    int_head(13), int_head(14)

* Step 9: compute response spectra

* Find time of acceleration maximum:
      call mnmaxidx( a,1,npts,1,
     :                 amin,amax,indx_min,indx_max) 
      imax = indx_min
      pga = abs(amin)
      if (abs(amax) .gt. abs(amin)) then
        imax = indx_max
        pga = abs(amax)
      end if
      tmax = float(imax - 1)*dt
      write(nu_sum,'(a)') 
     :     ' amin,amax,indx_min,indx_max,imax,tmax = '
      write(nu_sum,'(1p2(1x,e10.3),0p3(1x,i6),1p1x,e10.3)') 
     :       amin,amax,indx_min,indx_max,imax,tmax

* Loop over periods and damping, computing and writing response spectra:
      do iper = 1, nfosc
*        buf_out = ' '
*        buf_out = '+ Patience! Computing rd for ts '//
*     :  f_smc(1:nc_f_smc)//
*     :    ' and frequency '
*        write(buf_out(62:68),'(f7.3)') fosc(iper) 
*        write(*,'(a)') buf_out

*        write(*,'(a,1x,i3,1x,f7.3)') 
*     :     ' iper, fosc = ', iper, fosc(iper)

        omega = twopi*fosc(iper)
        omega2(iper) = omega**2
        damp = 0.05
        call rdrvaa(a, npts, omega, 
     :                   damp, dt, 
     :                   rd(iper), rv(iper), aa(iper), d0, v0)
      end do

      deallocate (a)

      v30 = 0.0

      write(nu_subnew,'(
     :    1x,a10, 1x,f4.0, 1x,f3.1, 2x,a6,
     :    5x,a1,2f6.0,
     :    1p,8(1x,e9.2),
     :    0p,1x,f6.2, 1x,f7.2, 1x,f3.1, 1x,a3)')
     :    eq_id, eqdepth, eqmag, f_smc(1:index(f_smc,'.')-1), 
     :    'U',  v30, rcd2f,
     :    pga, (omega2(i)*rd(i),i=1,7), 
     :    eqlat, eqlong, eqmag, ctype

      goto 1000 ! go back for another smc file

      end
* ---------------------------------------------------------------- SMC2Subd

      include '\forprogs\rdrvaa.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\skip.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\rcc.for'
      include '\forprogs\rcf.for'
      include '\forprogs\tabs_rmv.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\distaz.for'

