! ------------------------------------------------------- smc_hicut_fd_cosine_taper
      Program smc_hicut_fd_cosine_taper


! Input from a control file; here is an example:
!!Control file for program smc_hicut_fd_cosine_taper
!! ("fd" refers to the filtering being done in the frequency domain)
!! As many comment lines as desired, each starting with "!"
!! The string "pp:" indicates a new set of processing parameters
!! to be applied to the following smc files.  The parameters are given on the
!! following lines.
!! Revision of program involving a change in the control file on this date:
!   07/01/13
!!Name of summary file:
! smc_hicut_fd_cosine_taper.sum
!!mean info:
!! type of mean (0=no correction, 1=standard average; -1=specify mean)
!! t4mean_strt, t4mean_stop, mean (use if type = -1)
!! NOTE: need entries even if the parameter is not used (because read
!! info use list-directed [unformatted] input) 
!! Note that if working with a processed record that includes the 
!! complete time series produced by the processing (e.g., the padded time series
!! for acausal filtering or the post-processing baseline corrected time series
!! produced from pad-stripped or pass-through data in the first NGA project), 
!! it is inadvisible to remove the mean as this may 
!! produce unintended distortions at long periods; in this case set t4mean < 0.0.
!! If the record is pad-stripped, however, as provided by most
!! data centers, without post-processing to remove drifts in displacement,
!! a mean determined from the overall record probably should be 
!! removed.  But in this case the displacement will probably be inaccurate
!! (compared to the displacement derived from the padded, filtered acceleration
!! time series), it may be that a better approach is to specify the initial
!! velocity and displacement of the record---I am working on that (11/26/10):
! 0  1.0 20.0  -999.
!!f1, f2, and eta for high cut filter (1.0/2.0**eta)*(1+cos(pi*(f-f1)/(f2-f1)))**eta:
! 16.0 20.0 2.0  ! Used for simulations in Boore and Goulet (2014)
!!dc_remove
! F
!!taper_front, taper_back 
! 0.0 0.0 
!!String to append to input file name when constructing the output filename.
! .hicut_17_20hz.smc
!!List of files 
!pyls.smc.rm.a
!us.ncb.bhe.cor.acc.snipped.smc
!us.ncb.bhn.cor.acc.snipped.smc
!us.ncb.bhz.cor.acc.snipped.smc
!Stop
  
! Dates: 09/16/12 - Written by D.M. Boore, patterned after smc_interpolate_time_series_using_fft.for
!        06/30/13 - Use Fortran 95 extension getcl to get a command line argument that determines if
!                   the default control file name is used (this will be used for calling the main
!                   program within a batch file (c:\forprogs\smc_hicut_fd_cosine_taper -default_ctl)
!        07/01/13 - Generalize response to allow the raised cosine to be raised to a power (this
!                   fit better FAS of some data).
!        12/01/16 - Bring in subroutines through a single file (smc_hicut_fd_cosine_taper_subroutines.for),
!                   made by calling make_hicut_fd_cosine_taper_subroutines_file.bat and 
!                   make_smc_hicut_fd_cosine_taper_subroutines_file.bat in the 
!                   compile batch file cl_smc_hicut_fd_cosine_taper.bat.  The batch file 
!                   make_hicut_fd_cosine_taper_subroutines_file.bat makes subroutines used in the subroutine
!                   hicut_fd_cosine_taper (this sounds confusing, but I wanted to keep separate the utility-like
!                   subroutines used in the main program and the fft subroutines used in the 
!                   subroutine hicut_fd_cosine_taper.for).
!        12/05/16 - Add t_in_fft, data_length, df, fnyq_in, fupper_in_fft, n1, n2, npw2 to output
!                   of hicut_fd_cosine_taper, and write to f_sum
!                   
!
! Dimension and declaration statements:

      real ts_in(:), ts_out(:)
      allocatable :: ts_in, ts_out

      real tend, eta
      real*8 cum_avg
      
      integer status

      character date_c*8, time_c*11, extension_c*50, cl_arg*50

      character date_ctl_correct*8, date_ctl_in*30

      character char_head(11)*80, cmnts2skip(50)*80
      character comments(500)*80 
!      character comments(500)*200 

      real real_head(50)
      integer int_head(48)

      character buf*400, buf_upper*4, 
     :        f_sum*400, f_a*400,  f_out*400

      character f_ctl*200, precision*20
      logical f_exist, f_a_exist, rmv_trnd, 
     :  dc_remove
     
      f_exist = .false.
      cl_arg = ' '
      call getcl(cl_arg)
      call trim_c(cl_arg, nc_cl_arg)
      if (nc_cl_arg > 0) then
        f_ctl = ' '
        f_ctl = 'smc_hicut_fd_cosine_taper.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
          if (.not. f_exist) then
            write(*,'(a)') 
     :             ' ******* CONTROL FILE '//f_ctl(1:nc_f_ctl)//
     :             ' DOES NOT EXIST; QUITTING! ******* '
            stop
          end if
      else 
        do while (.not. f_exist)
          f_ctl = ' '
          write(*, '(a\)') 
     :      ' Enter name of control file '//
     :      '(Enter=smc_hicut_fd_cosine_taper.CTL;'//
     :      ' ctl-brk to quit): '
          read(*, '(a)') f_ctl
          if (f_ctl(1:4) .eq. '    ') 
     :      f_ctl = 'smc_hicut_fd_cosine_taper.ctl'
          call trim_c(f_ctl, nc_f_ctl)
          inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
          if (.not. f_exist) then
            write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
          end if
        end do
      end if

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '07/01/13'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') 
     :   ' Output of program test_interpolate_time_series_using_fft:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) fcut1, fcut2, eta

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      buf = ' '
      read(nu_ctl,'(a)') buf
      call upstr(buf)
      call trim_c(buf, nc_buf)
      write(nu_sum,'(a)') buf(1:nc_buf)
      if (buf(1:1) == 'T') then
        dc_remove = .true.
      else
        dc_remove = .false.
      end if
      write(nu_sum,'(a,1x,l1)') 
     :  ' dc_remove = ', dc_remove
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      buf = ' '
      read(nu_ctl,*) taper_front, taper_back 
      write(nu_sum,'(a)') 
     :  ' taper_front, taper_back = '
      write(nu_sum,*) 
     :    taper_front, taper_back 
      
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      extension_c = ' '
      read(nu_ctl,'(a)') extension_c
      call trim_c(extension_c, nc_extension_c)
      write(nu_sum,'(a)') 
     :  ' extension_c ='//extension_c(1:nc_extension_c)
 
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      num = 0
      
      loop over files: DO
      buf = ' '
      read(nu_ctl,'(a)',iostat=status) buf
      if(status /= 0) then
        EXIT loop over files
      end if
      call trim_c(buf,nc_buf)
      buf_upper = ' '
      buf_upper = buf(1:4)
      call upstr(buf_upper)
      if(nc_buf == 0 .or. buf(1:1) == '!') then
        CYCLE loop over files
      end if
      if(buf_upper(1:4) == 'STOP') then
        EXIT loop over files
      end if
      
      f_a = ' '
      f_a = buf(1:nc_buf)
      nc_f_a = nc_buf 
      
      f_a_exist = .false.
      inquire(file=f_a(1:nc_f_a), exist=f_a_exist)
!      write(*,'(a)') ' f_a: ', f_a(1:nc_f_a)
!      write(*,'(a, 1x, l1)') ' f_a_exist = ', f_a_exist

      if (.not. f_a_exist) then
          write(*,'(a)') ' ****** FILE '//f_a(1:nc_f_a)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE loop over files
      end if

! Passed tests; proceed

      num = num + 1

! Get number of points on this loop:

      call smc_npts(f_a(1:nc_f_a), npts_in, sps_in)

      sps_out =  sps_in
      npts_out = npts_in

      allocate( ts_in(npts_in), ts_out(npts_out) )

! Now read the data from the files:

!      write(*,'(1x,a)') 
!     :   'Processing file:', f_a(1:nc_f_a)

      tskip = 0.0
      tlength = -1.0

      call SMCRead(f_a(1:nc_f_a),  
     :               tskip, tlength,
     :               ts_in,  
     :               char_head, int_head, real_head, comments)
     
      precision = ' '
      if (int_head(47) == 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      call trim_c(precision,nc_precision)
     
 
      rnull = real_head(1)
       
      sps_in = real_head(2)
      ncomments_a = int_head(16)
      npts_in = int_head(17)
      dt_in = 1.0/sps_in
      tend_in = real(npts_in - 1)*dt_in

 
! Step 1: Remove a mean from the whole record

      t4mean_strt = t4mean_strt_in
      t4mean_stop = t4mean_stop_in
        
!  Determine the appropriate mean:
      if (itype_mean == 0) then
        n_t4mean = 0
        avg = 0.0
      else if (itype_mean == -1) then   ! use specified mean
        avg = spec_mean
      else if (itype_mean == 1) then
        if (t4mean_strt < 0.0) t4mean_strt = 0.0
        if (t4mean_stop > tend_in) t4mean_stop = tend_in
        n_t4mean_strt = nint(sps_in * t4mean_strt + 1.1)
        n_t4mean_stop = nint(sps_in * t4mean_stop + 1.1)
        n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
        t4mean = float(n_t4mean -1)/sps_in
!        write(nu_sum,'(a,1x,f6.2)') ' t4mean = ', t4mean
        cum_avg = 0.0d0
        do i = n_t4mean_strt, n_t4mean_stop
          cum_avg = cum_avg + dble(ts_in(i))
        end do
        avg = sngl( cum_avg/dble( real(n_t4mean) ) )
      else            ! shouldn't be here!
        write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
        deallocate (ts_in, ts_out)
        EXIT loop over files
      end if

!  Remove the mean from the whole record:
      do i = 1, npts_in
        ts_in(i) = ts_in(i) - avg
      end do

! Step2: High-cut filter

      call hicut_fd_cosine_taper(
     : ts_in, sps_in, npts_in, 
     : dc_remove, taper_front, taper_back, fcut1, fcut2, eta,
     : ts_out, sps_out, npts_out,
     : t_in_fft, data_length, df, fnyq_in, fupper_in_fft, n1, n2, npw2)

      write(nu_sum,*)
      write(nu_sum,'(1x,a)') ' sps_in, npts_in, '
      write(nu_sum, *)         sps_in, npts_in
      write(nu_sum,'(1x,a)') ' dc_remove, taper_front, taper_back'
      write(nu_sum,*)          dc_remove, taper_front, taper_back
      write(nu_sum,'(1x,a)') ' fcut1, fcut2, eta, sps_out, npts_out'
      write(nu_sum,*)          fcut1, fcut2, eta, sps_out, npts_out
      write(nu_sum,'(1x,a)') ' t_in_fft, data_length, df, fnyq_in'
      write(nu_sum,*)          t_in_fft, data_length, df, fnyq_in
      write(nu_sum,'(1x,a)') ' fupper_in_fft, n1, n2, npw2'
      write(nu_sum,*)          fupper_in_fft, n1, n2, npw2

! Step 3: Write output:

! Write output:
       if (nc_extension_c > 0) then
         f_out = f_a(1:nc_f_a)//extension_c(1:nc_extension_c)
       else
         print *,' ERROR, nc_extension_c = 0; QUITING'
       end if
       call trim_c(f_out, nc_f_out)

!        write(*,'(1x,a)') 
!     :   'Writing output to file:', f_out(1:nc_f_out)

        iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
        call datetime(date_c, time_c, iflag)

        ncomments = ncomments_a
        ncomments = ncomments + 1
        comments(ncomments) = '|'

        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| Interpolation of input file '//f_a(1:nc_f_a)
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '| using program smc_fd_hicut_cosine_taper'
        ncomments = ncomments + 1
        comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,2(1x,f7.3),a)') 
     :      '|   - cosine taper from = ', fcut1, fcut2, ' Hz'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a, 1x,f7.3)') 
     :      '|   - raised to the power = ', eta

        if (itype_mean == 0) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
        else if (itype_mean == 1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
        else if (itype_mean == -1) then
          ncomments = ncomments + 1
          write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
          ncomments = ncomments + 1
          write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
        else
          write(*,'(a)') ' Not supposed to be here.'
        end if

        ncomments = ncomments + 1
        comments(ncomments) = '|   resulting in output file '//
     :      f_out(1:nc_f_out)

        ncomments = ncomments + 1
        comments(ncomments) = '|'

        int_head(16) = ncomments 
        itype = 1

        real_head(2) = sps_out
        int_head(17) = npts_out
 
        call SMCWrite(f_out, precision,
     :                  ts_out, char_head, itype, 
     :                    int_head, real_head, comments)
     
 
        deallocate (ts_in, ts_out)

      END DO loop over files

! Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
! ------------------------------------------------------- smc_hicut_fd_cosine_taper

      include '\forprogs\smc_hicut_fd_cosine_taper_subroutines.for'
        !This file is made by calling make_smc_hicut_fd_cosine_taper_subroutines_file.bat
        !from within cl_smc_hicut_fd_cosine_taper.bat
        !One of the subroutines is hicut_fd_cosine_taper.for, which itself includes a suite
        !of subroutines named hicut_fd_cosine_taper_fft_subroutines.for, which is created
        !by calling make_hicut_fd_cosine_taper_fft_subroutines_file.bat
        !from within cl_smc_hicut_fd_cosine_taper.bat


!      include '\forprogs\skip.for'
!      include '\forprogs\skipcmnt.for'
!      include '\forprogs\get_lun.for'
!      include '\forprogs\upstr.for'
!      include '\forprogs\smc_npts.for'
!      include '\forprogs\smcread.for'
!      include '\forprogs\smcwrite.for'
!      include '\forprogs\distaz.for'
!      include '\forprogs\imnmax.for'
!      include '\forprogs\trim_c.for'
!      include '\forprogs\rc_subs.for'
!      include '\forprogs\datetime.for'
!      include '\forprogs\rmvtrend.for'
!      include '\forprogs\mnmaxidx.for'
!      include '\forprogs\hicut_fd_cosine_taper.for'

