! ----------------------------------------------------- smc_interpolate_time_series_using_fork
      Program smc_interpolate_time_series_using_fork


! Interpolates an smc file to finer spacing, using an FFT with zeroes
! added after the Nyquist.

! The new file name has "sic" rather than "smc" as the extension

! Input from a control file; here is an example:
!!Control file for program smc_interpolate_time_series_using_fork
!! As many comment lines as desired, each starting with "!"
!! Revision of program involving a change in the control file on this date:
!   09/29/12
!!Name of summary file:
! smc_interpolate_time_series_using_fork.sum
!!Interpolate factor (this MUST be a power of two; e.g., 2, 4, 8, etc.):
!! sps_out = interpolate_factor* sps_in
! 4
!!String to append to input file name when constructing the output filename.
! .fft_interp_factor_4.smc
!!
!!List of files
! file1.smc
! file2.smc
! file3.smc
!stop
 
! Dates: 10/19/02 - Written by D.M. Boore, patterned after SMC2FS2
!        10/21/02 - Added output filename to headers
!        07/17/03 - Add sps to smc_npts parameter list
!        02/28/05 - Add option of adding tag to output filename; 
!                   make compatible with LF95
!        07/05/06 - If desired, use linear interpolation to give values at
!                   a specified sampling rate.  This might be useful if
!                   want to compare two time series with different sample
!                   rates.  In such as case, the power of two for the
!                   interpolation in smcintrp should be set so that time series 1
!                   is oversampled (greater than the 
!                   rate of the second time series), and then linear
!                   interpolation will be used to downsample the time series.
!                   NOTE: the final sps should be less than the interpolated sps
!        10/17/07 - Transparent change to user needed because smcread can
!                   read two input formats
!        09/29/12 - Renamed from smcintrp and updated code syntax
!                 - Use a subroutine for the computation
!                 - Do not allow for linear resampling after interpolation;
!                   for that, see smcintrp
!        06/30/13 - Use Fortran 95 extension getcl to get a command line argument that determines if
!                   the default control file name is used (this will be used for calling the main
!                   program within a batch file (c:\forprogs\smc_hicut_fd_cosine_taper -default_ctl)

      character precision*8


!
! Dimension and declaration statements:

      real y_in(:), y_out(:) 
      allocatable :: y_in, y_out 

      character date_c*8, time_c*11

      character char_head(11)*80, comments(500)*80, cmnts2skip(50)*80
      real real_head(50)
      integer int_head(48)
      
      integer ios

      character date_ctl_correct*8, date_ctl_in*30

      character buf*80, buf4*4, 
     :        f_in*300, f_out*300, f_sum*300, tag*100, cl_arg*50

      character f_ctl*200
      logical f_exist, f_in_exist

      f_exist = .false.
      cl_arg = ' '
      call getcl(cl_arg)
      call trim_c(cl_arg, nc_cl_arg)
      if (nc_cl_arg > 0) then
        f_ctl = ' '
        f_ctl = 'smc_interpolate_time_series_using_fork.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
          if (.not. f_exist) then
            write(*,'(a)') 
     :             ' ******* CONTROL FILE '//f_ctl(1:nc_f_ctl)//
     :             ' DOES NOT EXIST; QUITTING! ******* '
            stop
          end if
      else 
        do while (.not. f_exist)
          f_ctl = ' '
          write(*, '(a\)') 
     :      ' Enter name of control file '//
     :      '(Enter=smc_interpolate_time_series_using_fork.ctl;'//
     :      ' ctl-brk to quit): '
          read(*, '(a)') f_ctl
          if (f_ctl(1:4) .eq. '    ') 
     :      f_ctl = 'smc_interpolate_time_series_using_fork.ctl'
          call trim_c(f_ctl, nc_f_ctl)
          inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
          if (.not. f_exist) then
            write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
          end if
        end do
      end if

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip,nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '09/29/12'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)

      if (date_ctl_correct(1:nc_date_ctl_correct) .ne. 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMCIntrp:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)
      print *, '   Control file = '//f_ctl(1:nc_f_ctl)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :   interpolate_factor  ! This must be a power of two--check
       if (modulo(alog(real(interpolate_factor)), alog(2.0)) /= 0) then
         print *,' interpolate_factor= ', 
     :              interpolate_factor, ' not a power of 2; QUIT!!'
         close(nu_ctl)
         stop
       end if

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      tag = ' '
      read(nu_ctl,'(a)') tag
      call trim_c(tag,nc_tag)
      write(nu_sum,'(a,1x,i2)') ' nc_tag = ', nc_tag
      print *, ' nc_tag = ', nc_tag
      if (nc_tag > 0) then
        write(nu_sum,'(a)') ' tag = '//tag(1:nc_tag)
        print *,' tag = '//tag(1:nc_tag)
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      
      loop over input files: DO
        buf = ' '
        read(nu_ctl,'(a)',iostat=ios) buf
        if (ios /= 0) then
          EXIT loop over input files
        end if
          
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call upstr(buf4)
        if(buf4(1:4) == 'STOP') then
          EXIT loop over input files
        end if
       
        f_in = ' '
        f_in = buf(1:nc_buf)
        nc_f_in = nc_buf     
        inquire(file=f_in(1:nc_f_in), exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ****** FILE '//f_in(1:nc_f_in)// 
     :          ' DOES NOT EXIST; SKIPPING!!! ******* '
          CYCLE loop over input files
        end if

! Passed tests; proceed
 
! Get number of points on this loop:

      call smc_npts(f_in(1:nc_f_in), npts_in, sps_in)

      write(nu_sum,*)
      write(nu_sum, '(a, 2(1x,i5))') ' npts_in = ', npts_in

      sps_out =  real(interpolate_factor) * sps_in
      npts_out = interpolate_factor * npts_in

      allocate( y_in(npts_in), y_out(npts_out) )

! Now read the data from the files:

      write(*,*)
      write(*,'(1x,a)') 
     :   'Processing file:', f_in(1:nc_f_in)

      tlength = -1.0 ! read all data
      tskip = 0.0    ! don't skip any data before starting to read
      call SMCRead(f_in(1:nc_f_in), tskip, tlength,
     :               y_in,  
     :               char_head, int_head, real_head, comments)
      precision = ' '
      if (int_head(47) == 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
      sps_in = real_head(2)
      npts_in = int_head(17)
      dt_in = 1.0/sps_in
      t_in = float(npts_in)*dt_in

      write(nu_sum,'(3x,a,f8.6,a,i5,a,f8.2)') 
     :     'signal_in: dt_in=', dt_in, 
     :     ' npts_in=', npts_in,' t_in=', t_in

      print *, 
     :     'signal_in: dt_in=', dt_in, 
     :     ' npts_in=', npts_in,' t_in=', t_in

      call interpolate_time_series_using_fork(
     : y_in, sps_in, npts_in, 
     : interpolate_factor,
     : y_out, sps_out, npts_out
     : )
  
! Write output:

! Construct output filename:
      f_out = ' '
      f_out = f_in(1:nc_f_in)
      if (nc_tag > 0) then
        f_out = f_in(1:nc_f_in)//tag(1:nc_tag)
      end if
      call trim_c(f_out, nc_f_out)

      write(*,'(1x,a)') 
     :   'Writing output to file:', f_out(1:nc_f_out)
      write(nu_sum,'(2a)') ' Output file = ', f_out(1:nc_f_out)

      ncomments = int_head(16)
      ncomments = ncomments + 1
      comments(ncomments) = '|'
      ncomments = ncomments + 1
      comments(ncomments) = '|  Input file '//f_in(1:nc_f_in)
      ncomments = ncomments + 1
      comments(ncomments) = '|  resampled using program '//
     :          'smc_interpolate_time_series_using_fork'
      iflag = 0
      call datetime(date_c, time_c, iflag)
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|  file '//f_out(1:nc_f_out)//
     :      ' written on '//date_c//' at time '//time_c
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   sps_in, sps_out = '
      write(comments(ncomments)(23:46),
     :      '(3(1x,f7.2))') sps_in, sps_out 
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      int_head(16) = ncomments
      real_head(2) = sps_out
      int_head(17) = npts_out

      itype = -1  ! use info from header
      call SMCWrite(f_out(1:nc_f_out), precision,
     :                y_out, char_head, itype, 
     :                      int_head, real_head, comments)

      deallocate (y_in, y_out)
      
      END DO loop over input files

! Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
! ----------------------------------------------------- smc_interpolate_time_series_using_fork


      include '\forprogs\fork.for'
!      include '\forprogs\forkdp.for'
      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\get_npw2.for'
      include '\forprogs\datetime.for'
      include '\forprogs\downsample.for'
      include '\forprogs\distaz.for'
      include '\forprogs\interpolate_time_series_using_fork.for'

