! ----------------------------------------------- SMC_PSA_vs_Rotation_Angle
      Program SMC_PSA_vs_Rotation_Angle

* Reads a pair of SMC-format filenames from a control file and computes
* and writes the PSA for a specified period as a function of a suite of rotation
* angles. The purpose is to check results from smc2psa_rot_gmrot or nga2psa_rot_gmrot.
 
* Structure of control file:
* Line 1 follows:
*!Control file for program SMC_PSA_Vs_Rotation_Angle
*! Revision of program involving a change in the control file on this date:
*   01/24/10
*!Summary file name:
* smc_psa_vs_rotation_angle.sum
*!d_rot_angle
*  1.0 
*!Type of response spectrum (0=SD, 1=PSV, 2=PSA)
*  0
*!Damping value:
* 0.05
*!Number of periods:
*  4
*!Periods (<0 for pgv, 0 for pga):
* 0.1
* 0.2
* 1.0
* 3.0
*!t4mean (removes a mean from the whole record, where the mean is
*!    determined from t = 0 to t = t4mean.  If t4mean = 0.0,
*!    the mean of the whole record is used, and if t4mean < 0, no mean is
*!    removed.  Note that if working with a processed record, it is
*!    inadvisible to remove the mean as this may produce unintended
*!    distortions at long periods --- so in most cases set t4mean < 0.0):
* -10.0  
*! Skip analysis (including rotation) if npts differ by more than:
*  2
*!Stem name of output file:
* smc_psa_vs_rotation_angle
*! list of file names (assumes pairs of files) 
*    0516a.smc
*    0516c.smc
*    0526a.smc
*    0526c.smc
*  StoP
 
* Sign conventions:  the azimuth of the radial component is 180+azimuth from
* station to reference point, and the transverse component is this radial 
* azimuth plus 90 degrees.  This should be equivalent to the convention
* in Aki and Richards, Fig. 4.10, p. 96.

* This is incomplete, because the program will eventually also deal with case
* for which i_spcfy_fname = 1.  A program that works 
* with this is smcrtold (the old version of smc_rot)

* Dates: 01/24/10 - Written by D. Boore, based on smc2psa_rot_gmrot. 
*        02/13/10 - Add type of response spectrum
!        09/30/10 - Substituted RSCALC_INTERP_ACC for RD_CALC
!        10/06/11 - Increase number of angles
!        12/12/11 - Change "ctl_cmmnts" to "cmnts2skip"
!        02/28/18 - Substituted RS_CALC for RSCALC_INTERP_ACC
                
      character char_head(11,2)*80, comments(200,2)*80, buf_file*100
      character cmnts2skip(20)*80, path_in*40, path_out*40      
       
      character date_ctl_correct*8, date_ctl_in*30

      integer idate_time_values(8)
      
      real time_sec_start, time_sec_stop

      integer int_head(48,2), nc_f_in(2),  nc_f_out, nc_rc(2)
      integer npts(2)

      real real_head(50,2), work(5000), work_rotset(1000), 
     :     work_angle4rot(1000), rot_angle4gmrot(1000)
      real alat_sta(2), along_sta(2)
      real a_in(:,:), a_out(:,:) 
      allocatable :: a_in, a_out 
      
      real period(300), rd(2), rs(2)
      real rs_rotset(1000,300), rot_angle4rot(1000) !1000=nrot, 300 = nper
      real omega, omega2
      real per4az(10)
      integer ndx4az(10)
      integer index_rotset(1000)
      
      real flc(2)
      integer nslope(2)
      
      integer nper_record(5000)
      real iflag4rec(5000)
      
      character per_c*10, colhead(20)*12
         

      real rnull(2), azm_in(2), azm_out(2), azm4file(200), sps(2)
      character buf*80, buf4*4, f_ctl*80, f_sum*80,
     :          buf_out*79
      character  f_in(2)*200, f_out_stem*100, f_out*200, 
     :           f_ext*200, f_penalty*100,
     :           c_az*7, rc(2)*12
      character  frmt_c*80

      logical f_exist
      logical rmv_trnd, odd_l, use_straightline_interp
      
 
      pi = 4.0*atan(1.0)
      twopi = 2.0*pi
      d2r = pi/180.0
      nper_dimen = 300
      nrot_dimen = 1000
      
      f_exist = .false.
      do while (.not. f_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file '//
     :    '("Enter" = smc_psa_vs_rotation_angle.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'smc_psa_vs_rotation_angle.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_exist)
        if (.not. f_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

! Check date of control file:
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      date_ctl_in = ' '
      read(nu_ctl,'(a)') date_ctl_in
      call trim_c(date_ctl_in,nc_date_ctl_in)
      
      date_ctl_correct = ' '
      date_ctl_correct = '01/24/10'
      call trim_c(date_ctl_correct,nc_date_ctl_correct)


      if (date_ctl_correct(1:nc_date_ctl_correct) /= 
     :    date_ctl_in(1:nc_date_ctl_in)) then
        write(*,'(a)') 
     :     ' The control file has the wrong date; update your '//
     :       'control file and rerun the program!'
        write(*,'(a)') 
     :     ' The date for the control file you used is '
     :       //date_ctl_in(1:nc_date_ctl_in)
        write(*,'(a)') 
     :     ' The correct date is '
     :       //date_ctl_correct(1:nc_date_ctl_correct)
        close(nu_ctl)
        stop
      end if
! Check date of control file:
       
      call skipcmnt(nu_ctl,cmnts2skip, nc_cmnts2skip)
      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call get_lun(nu_sum)
      call trim_c(f_sum,nc_f_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')    
      write(nu_sum,'(a)') 
     :  ' Summary file for program smc_psa_vs_rotation_angle'
      write(nu_sum,'(a)') '   Using control file: '//f_ctl(1:nc_f_ctl)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(a)') cmnts2skip(i)
      end do
      write(nu_sum,'(a)') f_sum

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) d_rot_angle

      rot_low = 0.0
      rot = rot_low
      nrot = 1
      do
        rot = rot + d_rot_angle
        if (rot .ge. 90.0) exit
        rot_high = rot
        nrot = nrot + 1
      end do
      
      write(nu_sum,'(a)') 
     :  ' d_rot_angle, nrot, rot_high, rot_high_by_comp = ' 
      write(nu_sum,*) 
     :    d_rot_angle, nrot, rot_high, float(nrot - 1)*d_rot_angle
      if (nrot .gt. nrot_dimen) then
        nrot = nrot_dimen
        d_rot_angle = 90.0/float(nrot)
        write(nu_sum,'(a,1x,i3)') ' resetting nrot to ', nrot
        write(nu_sum,'(a,1x,i3)') ' and d_rot_angle to ', d_rot_angle
      else        
        write(nu_sum,'(a,1x,i3)') ' nrot = ', nrot
      end if
      
      n_rotset = 2*nrot
      write(nu_sum,'(a, 1x,i4, 1x,i4)')  
     :  ' n_rotset, nrot_dimen = ', n_rotset, nrot_dimen 
      if (n_rotset .gt. nrot_dimen) then
        print *,' ERROR: n_rotset > nrot_dimen; QUIT!!!'
        stop
      end if

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) idva

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) damp
      write(nu_sum,*) damp
      
       
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) nperiods
      write(nu_sum,*) nperiods
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) (period(i), i=1,nperiods)
      write(nu_sum,*) (period(i), i=1,nperiods)
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) t4mean_in
      write(nu_sum,*) t4mean_in
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      read(nu_ctl,*) npts_skip
      write(nu_sum,*) npts_skip
         
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      do i = 1, nc_cmnts2skip
        write(nu_sum,'(1x,a)') cmnts2skip(i)(1:79)
      end do
      f_out_stem = ' '
      read(nu_ctl,'(a)') f_out_stem
      call trim_c(f_out_stem,nc_f_out_stem)
      write(nu_sum,'(a)') f_out_stem(1:nc_f_out_stem)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      
      nrecs_all = 0

      loop_over_files: DO

        write(nu_sum,*)

* First get max dimension and allocate arrays:
        do j = 1, 2
          buf = ' '
          read(nu_ctl,'(a)') buf
          call trim_c(buf,nc_buf)
          print *,' buf = '//buf(1:nc_buf)
          buf4 = ' '
          buf4 = buf(1:4)
          call upstr(buf4)

          if (buf4(1:4) .eq. 'STOP') EXIT loop_over_files

          f_in(j) = ' '
          f_in(j) = buf(1:nc_buf)
          call trim_c(f_in(j), nc_f_in(j))
          f_exist = .false.
          inquire(file=f_in(j)(1:nc_f_in(j)), exist=f_exist)
          if (.not. f_exist) then
            write(nu_sum,'(a)') ' ******* FILE '//
     :                         f_in(j)(1:nc_f_in(j))//
     :                        ' DOES NOT EXIST: QUIT! ******* '
            EXIT
          end if
          
          write(nu_sum,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
          write(*,'(a,i1,a)') 
     :                 ' f_in(',j,') = '//f_in(j)(1:nc_f_in(j))
     
          call smc_npts(f_in(j)(1:nc_f_in(j)), npts(j), sps(j))
          
        end do	! end reading of two files from control file
        
        call date_and_time(values=idate_time_values)
        time_sec_start = float(idate_time_values(5)*60*60+
     :                           idate_time_values(6)*60   +
     :                           idate_time_values(7))     +
     :                     float(idate_time_values(8))/1000.0
     
        max_dimen = max0( npts(1), npts(2) )
        allocate( a_in(max_dimen,2), a_out(max_dimen,2) )


* Second, loop over input files, reading data:
        read_data_both_components: DO j = 1, 2
        
          call SMCRead(f_in(j)(1:nc_f_in(j)), 0.0, 100000.0,
     :                 a_in(1,j), char_head(1,j), 
     :                 int_head(1,j), real_head(1,j), 
     :                 comments(1,j))

          npts(j)  = int_head(17,j)  ! should check equality of both time series
          rnull(j) = real_head(1,j)
          azm_in(j) = real(int_head(14,j)) 
          if (real_head(33,j) .eq. rnull(j)) then
            flc(j) = -9.9
          else
            flc(j) = real_head(33,j)
          end if
          if (real_head(34,j) .eq. rnull(j)) then
            nslope(j) = -9
          else
            nslope(j) = 4*int(abs(real_head(34,j)))  ! assume acausal
          end if
 
          if (j .eq. 1) then   ! overwrite any existing azimuth in the header with 0 and 90
            azm_in(j) = 0.0
          else
            azm_in(j) = 90.0
          end if
          
          azm_out(j) = azm_in(j)  ! needed because Rotate overwrites azm_out
                                  ! in argument list

          do i = 1, npts(j)
            a_out(i,j) = a_in(i,j)
          end do
          
        END DO read_data_both_components  ! loop over reading data for the two components 

 
* Check for equality of sps:
        if ( sps(1) .ne. sps(2)) then
            write(nu_sum,'(a,1p, 2(1x,e12.5))') 
     :      ' SKIPPING: sps1 .ne. sps2; sps1, sps2 = ', 
     :        sps(1), sps(2)
          deallocate( a_in ) 
          deallocate( a_out )
          CYCLE
        end if
        
* Check for equality of npts:
        if ( iabs(npts(1) - npts(2)) .gt. npts_skip) then
            write(nu_sum,'(a,3(1x,i6))') 
     :      ' SKIPPING: iabs(npts1 - npts2) > '//
     :      'npts_skip; npts1, npts2, npts_skip = ', 
     :        npts(1), npts(2), npts_skip
          deallocate( a_in ) 
          deallocate( a_out )
          CYCLE
        end if
 
* Check for azimuths being 90 degrees apart (compute the dot product):
        dot = cos(d2r*azm_in(1))*cos(d2r*azm_in(2))+
     :        sin(d2r*azm_in(1))*sin(d2r*azm_in(2))
        eps = 0.001
        if (abs(dot) .gt. eps) then
          write(*,'(a)') '  Dot product not 0.0; see SUM file'//
     :                   ' for details; SKIPPING!!!'
          write(nu_sum,'(a)') '  azm_in(1,2), dot = '
          write(nu_sum,*) azm_in(1), azm_in(2), dot
          deallocate( a_in ) 
          deallocate( a_out )
          CYCLE
        end if

* Passed all equality tests.

* Step: Remove a mean from the whole record

*  Determine the appropriate mean:
        do j = 1, 2
          t4mean = t4mean_in
          if(t4mean .lt. 0.0) then  ! do not remove a mean
            avg = 0.0
          else
            tend = float(npts(j)-1)/sps(j)
            if (t4mean .eq. 0.0 .or. t4mean .gt. tend) then   
              t4mean = tend  ! use whole record
            end if
          end if
          n_t4mean = sps(j) * t4mean + 1.1
          cum_avg = 0.0
          do i = 1, n_t4mean
            cum_avg = cum_avg + dble(a_in(i,j))
          end do
          avg = cum_avg/float(n_t4mean)
*          avg = sngl(cum_avg/n_t4mean)
*  Remove the mean from the whole record:
          do i = 1, npts(j)
            a_in(i,j) = a_in(i,j) - avg
          end do
        end do
 
! set nosc_ts (used in rotations) to minimum value of the npts for each time series 
! (and subtract 1: see rdcalcts).  If not done, the value of osc_ts_as_recorded(i,j) used 
! in computing osc_ts_rotated(i,j) below could be a mystery value if i exceeds the actual
! length of the jth time series.  Previously nosc-ts was set to npts(j) in the loop above,
! so that nosc_ts after the loop equalled npts(2).  If npts(2) > npts(1) there could be a
! problem (this may have produced some strange results that cropped up only rarely, when
! drotd50 was huge for a particular record when it was included in a long list of
! records, but not when the record as placed near the beginning of the control file).

          nosc_ts = min0(npts(1), npts(2)) - 1 
          
          write(nu_sum,'(a, 1x,i5, 1x,i5, 1x,i5)') 
     :      ' npts(1), npts(2), nosc_ts = ',          
     :        npts(1), npts(2), nosc_ts
     
          f_out = ' '
          f_out = f_out_stem(1:nc_f_out_stem)//'.'//
     :                  f_in(1)(1:nc_f_in(1))//'.out'
          call trim_c(f_out, nc_f_out)
          call get_lun(nu_out)
          open(nu_out,file=f_out(1:nc_f_out),status='unknown')


* Set up and write column headers of output file:

      do i = 1, nperiods
        per_c = ' '
        write(per_c,'(f9.5)') period(i)
        call trim_c(per_c,nc_per_c)
        colhead(i) = ' '
        if (idva == 0) then
          write(colhead(i)(1:12),'(a12)') 'SD0000000000'
        else if (idva == 1) then
          write(colhead(i)(1:12),'(a12)') 'PSV000000000'
        else if (idva == 2) then
          write(colhead(i)(1:12),'(a12)') 'PSA000000000'
        else
          write(*,*) ' Value of idva (', idva,') is invalid; QUITTING!'
          close(nu_ctl)
          stop
        end if
        
        colhead(i)(12-nc_per_c+1:12) = per_c(1:nc_per_c)
      end do
      
      frmt_c = ' '
      frmt_c = '(1x,a9, xx(1x,a12))'                                  !xx are characters 9 and 10
      call trim_c(frmt_c,nc_frmt_c)
*DEBUG
      write(*,*) ' nc_frmt_c, frmt_c(1:nc_frmt_c) = ',
     :             nc_frmt_c, frmt_c(1:nc_frmt_c)
*DEBUG
      write(frmt_c(9:10), '(i2.2)') nperiods
! If insert new variables, be careful about specifying column numbers
! corresponding to xx


      call trim_c(frmt_c,nc_frmt_c)
      
      write(nu_sum,'(a)') ' frmt_c = '//frmt_c(1:nc_frmt_c)


      write(nu_out,frmt_c(1:nc_frmt_c)) 
     :   'rot_angle',
     :    (colhead(i), i = 1, nperiods)
     

* Step: Loop over azimuths:

          psamaxrot = 0.0
          psaminrot = 99999.0

          loop_over_angles: DO iz = 1, nrot
        
            rot_angle4gmrot(iz) = float(iz-1)*d_rot_angle   ! note: iz = 1 corresponds to no rotation
            rot_angle4rot(2*(iz-1)+1) = rot_angle4gmrot(iz)
            rot_angle4rot(2*(iz-1)+2) = rot_angle4gmrot(iz) + 90.0  
         
            a_out = 0.0
            
            do j = 1, 2
              do i = 1, nosc_ts
                a_out(i,j) = a_in(i,j)
              end do
            end do
 
! To decrease possible confusion, I am "fooling" rotate by hardwiring the azimuths
! of components 1 and 2---for this application the actual azimuths are not important.
            azm1 = 0.0
            azm2 = 90.0
            znull = 1.7000000E+38
            call Rotate(a_out(1,1),a_out(1,2),
     :           znull,
     :           azm1,azm2,rot_angle4gmrot(iz),nosc_ts)
     
* Compute response spectra:

* Step: Loop over periods:

            DO iper = 1, nperiods
        
              buf_out = ' '
              buf_out = '+ Patience! Computing rd for'//
     :                  ' rot_angle4gmrot = '
              write(buf_out(43:47),'(f5.1)') rot_angle4gmrot(iz)
              buf_out = buf_out(1:47)//' and period '
              write(buf_out(61:68),'(f8.4)') period(iper) 
!              write(*,'(a)') buf_out

* Compute psa for this period:
              omega = twopi/period(iper)
              omega2 = omega**2
              use_straightline_interp = .true.
              DO j = 1, 2
                dt = 1.0/sps(j)
                d0 = 0.0
                v0 = 0.0
                call rs_calc(a_out(1,j), nosc_ts, omega, 
     :            damp, dt, 
     :            v0, d0, use_straightline_interp, 
     :            rd(j), rv_dum, aa_dum )
!                call rscalc_interp_acc(a_out(1,j), nosc_ts, omega, 
!     :            damp, dt, rd(j), rv_dum, aa_dum )
     
                rs(j) = rd(j) * omega**idva
                
                rs_rotset(2*(iz-1)+j,iper) = rs(j)
                
              END DO

            END DO ! loop over periods
        
* Write PSA for this angle:



          END DO loop_over_angles ! end loop over azimuths
          
        DO iz = 1, n_rotset      
          write(nu_out,'(5x,f5.1, 20(1x,es12.5))') 
     :    rot_angle4rot(iz), (rs_rotset(iz,iper), iper = 1, nperiods) 
        END DO
        close(nu_out)
     
        deallocate(a_in, a_out)

      END DO loop_over_files   ! loop back for another pair of components

      close(nu_sum)
      close(nu_ctl)
 
      stop
      end
! ----------------------------------------------- SMC_PSA_vs_Rotation_Angle

 
           
      include '\forprogs\rmvtrend.for'     
      include '\forprogs\rotate.for'     
      include '\forprogs\get_lun.for'     
      include '\forprogs\skip.for'     
      include '\forprogs\smcread.for'     
      include '\forprogs\mnmaxidx.for'     
      include '\forprogs\abs_mnmaxidx.for'     
      include '\forprogs\trim_c.for'     
      include '\forprogs\upstr.for'     
      include '\forprogs\smc_npts.for'     
      include '\forprogs\distaz.for'     
      include '\forprogs\rc_subs.for'     
      include '\forprogs\skipcmnt.for'     
      include '\forprogs\rs_calc.for'
      include '\forprogs\moment.for'
      include '\forprogs\select.for'
      include '\forprogs\indexx.for'
  

 