* ------------------------------------------------------------------- SMCIntY1Y2
      Program SMCIntY1Y2


* Produces a file of the cumulative integral square of two time series

* The output file name is in smc format, with the name constructed
* from the names of the input files.  For example, for
* the file names in the sample control file below, the output file
* would have a name:

* tcu1291u.smc.tcu1293u.smc.inty1y2
 
* Input from a control file; here is an example:
*!Control file for program SMCIntY1Y2
*! As many comment lines as desired, each starting with "!"
*!Name of summary file:
* smc2intsq2.sum
*!mean info:
*! type of mean (0=no correction, 1=standard average; -1=specify mean)
*! t4mean_strt, t4mean_stop, mean (use if type = -1)
*! NOTE: need entries even if the parameter is not used (because read
*! info use list-directed [unformatted] input) 
*! Note that if working with a processed record, it is
*! inadvisible to remove the mean as this may produce unintended
*! distortions at long periods --- so in most cases set t4mean < 0.0):
* 0  1.0 20.0  -999.
*!List of files (the two files to multiply and integrate)
* tcu1291u.smc
* tcu1293u.smc
*Stop
 
* Dates: 11/06/06 - Written by D.M. Boore, patterned after SMC2Husid
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats
*        02/13/09 - Correct error in t4mean_strt and t4mean_stop being reset from their input
*                   values when processing a list of files.  The problem is that if a file with
*                   a shorter record length is processed before one with a longer record
*                   length, a mean supposedly determined for the whole record was determined
*                   only for the length of the shorter preceeding record.

      character precision*8


c
c Dimension and declaration statements:

      real y(:,:), y1y2(:), y1y2_int(:)
      allocatable :: y, y1y2, y1y2_int

      integer npts_in(2), nc_f_ctl, nc_f_y(2), nc_f_out
      real tend, sps_in(2)
      real*8 cum_avg

      character date_c*8, time_c*11

      character char_head(11)*80, comments(500)*80, cmnts2skip(200)*80
      real real_head(50)
      integer int_head(48)

      character buf*200, buf4*4, 
     :        f_sum*200, f_y(2)*200, f_out*200

      character f_ctl*40
      logical f_ctl_exist, f_y_exist, rmv_trnd

      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr=SMCIntY1Y2.CTL;'//
     :    ' ctl-brk to quit): '
        read(*, '(a)') f_ctl
        call trim_c(f_ctl, nc_f_ctl)
        if (f_ctl(1:4) .eq. '    ') f_ctl = 'smcinty1y2.ctl'
        call trim_c(f_ctl, nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

      f_sum = ' '
      read(nu_ctl,'(a)') f_sum
      call trim_c(f_sum,nc_f_sum)
      call get_lun(nu_sum)
      open(unit=nu_sum,file=f_sum(1:nc_f_sum),status='unknown')

      write(nu_sum,'(a)') ' Output of program SMC2IntY1Y2:'
      write(nu_sum,'(2a)') '   Control file = ', f_ctl(1:nc_f_ctl)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)
      read(nu_ctl,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean
      write(nu_sum,'(a)') 
     :  ' itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean:'
      write(nu_sum,*) 
     :    itype_mean, t4mean_strt_in, t4mean_stop_in, spec_mean

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)

1000  continue

      print *

      do iy = 1, 2
         buf = ' '
         read(nu_ctl,'(a)',end = 9999) buf
         call trim_c(buf,nc_buf)
         buf4 = ' '
         buf4 = buf(1:4)
         call upstr(buf4)
         if(nc_buf .eq. 0 .or. buf(1:1) .eq. '!') go to 1000
         if(buf4 .eq. 'STOP') go to 9999
      
         f_y(iy) = ' '
         f_y(iy) = buf(1:nc_buf)
         nc_f_y(iy) = nc_buf     
         inquire(file=f_y(iy)(1:nc_f_y(iy)), exist=f_y_exist)
         if (.not. f_y_exist) then
            write(*,'(a)') ' ****** FILE '//f_y(iy)(1:nc_f_y(iy))// 
     :            ' DOES NOT EXIST; QUITTING!!! ******* '
            go to 9999
         end if

         print *,' Working on file: '//f_y(iy)(1:nc_f_y(iy))
         
* Get number of points on this loop:

         call smc_npts(f_y(iy)(1:nc_f_y(iy)), npts_in(iy), sps_in(iy))

         write(nu_sum,*)
         write(nu_sum, '(a, 2(1x,i5))') ' iy, npts = ', iy, npts_in(iy)
         
      end do
      
* Check for equality of npts and sps  

      if (npts_in(1) .ne. npts_in(2)) then
         write(*, '(a, 2(1x,i5))') 
     :        ' QUITTING, npts not equal: ', npts_in(1), npts_in(2)
         write(nu_sum, '(a, 2(1x,i5))') 
     :        ' QUITTING, npts not equal: ', npts_in(1), npts_in(2)
         go to 9999
      end if

      if (sps_in(1) .ne. sps_in(2)) then
         write(*, '(a, 2(1x,i5))') 
     :        ' QUITTING, sps not equal: ', sps_in(1), sps_in(2)
         write(nu_sum, '(a, 2(1x,i5))') 
     :        ' QUITTING, sps not equal: ', sps_in(1), sps_in(2)
         go to 9999
      end if

* OK to continue

      npts = npts_in(1)
      sps  = sps_in(1)
      dt = 1.0/sps

      allocate( y(npts+5,2), y1y2(npts+5), 
     :          y1y2_int(npts+5) )

      do iy = 1, 2

* Now read the data from the files:

         write(*,'(1x,a)') 
     :   'Processing file:', f_y(iy)(1:nc_f_y(iy))

         tskip = 0.0
         tlength = -1.0

         call SMCRead(f_y(iy)(1:nc_f_y(iy)), tskip, tlength,
     :               y(1,iy),  
     :               char_head, int_head, real_head, comments)

      precision = ' '
      if (int_head(47) .eq. 8) then
        precision = 'HIGH'
      else
        precision = 'STANDARD'
      end if
      
         rnull = real_head(1)
 
         ncomments_y = int_head(16)
         tend = float(npts-1)*dt



* Step 1: Remove a mean from the whole record

         t4mean_strt = t4mean_strt_in
         t4mean_stop = t4mean_stop_in
        
*  Determine the appropriate mean:
         if (itype_mean .eq. 0) then
            n_t4mean = 0
               avg = 0.0
            else if (itype_mean .eq. -1) then   ! use specified mean
               avg = spec_mean
         else if (itype_mean .eq. 1) then
            if (t4mean_strt .lt. 0.0) t4mean_strt = 0.0
            if (t4mean_stop .gt. tend) t4mean_stop = tend
            n_t4mean_strt = nint(sps * t4mean_strt + 1.1)
            n_t4mean_stop = nint(sps * t4mean_stop + 1.1)
            n_t4mean = n_t4mean_stop - n_t4mean_strt + 1
            t4mean = float(n_t4mean -1)/sps
            cum_avg = 0.0
            do i = n_t4mean_strt, n_t4mean_stop
               cum_avg = cum_avg + dble(y(i,iy))
            end do
            avg = sngl(cum_avg/n_t4mean)
         else            ! shouldn't be here!
            write(nu_sum, '(a,1x,i3,1x,a)') '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
            write(*) '  Should not be here, '//'
     :       improper value of itype_mean.  itype_mean = ', 
     :       itype_mean, ' QUITTING!!'
            deallocate (y, y1y2, y1y2_int)
            go to 9999
         end if

*  Remove the mean from the whole record:
         do i = 1, npts
            y(i, iy) = y(i, iy) - avg
         end do

      end do
      
* Step 2: Multiply the two time series

      do i = 1, npts
        y1y2(i) = y(i, 1) * y(i, 2)
      end do
      
      deallocate (y)
      
      
* Step 3: Integrate the product of y1 and y2

      rmv_trnd = .false.
      call integrate_y(y1y2, npts, dt, rmv_trnd, y1y2_int)
      
      deallocate (y1y2)
 
* Step 4: Write output:

      f_out = ' '
      f_out = f_y(1)(1:nc_f_y(1))//f_y(2)(1:nc_f_y(2))//'.inty1y2'
      call trim_c(f_out,nc_f_out)
      write(*,'(1x,a)') 
     :   'Writing smc2inty1y2 output to file:', f_out(1:nc_f_out)

      iflag = 0 ! not used by current PC version of datetime; Chris
                ! Stephens would modify datetime.for to include Unix routines
      call datetime(date_c, time_c, iflag)

      ncomments = ncomments_y
      ncomments = ncomments + 1
      comments(ncomments) = '|'

      ncomments = ncomments + 1
      comments(ncomments) = 
     :     '| Using program SMCIntY1Y2, formed the '
     :   //    'integral of the product of '//f_y(1)(1:nc_f_y(1))
     :   //    ' and '//f_y(2)(1:nc_f_y(2))
      ncomments = ncomments + 1
      comments(ncomments) = 
     :      '|   on '//date_c//' at time '//time_c

      if (itype_mean .eq. 0) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|   - no mean subtracted from record'
      else if (itype_mean .eq. 1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,f7.3,a,1x,f7.3)') 
     :      '|     mean determined from t = ', t4mean_strt, 
     :      ' to t = ', t4mean_stop
      else if (itype_mean .eq. -1) then
        ncomments = ncomments + 1
        write(comments(ncomments),'(a,1x,1pe12.5,a)') 
     :      '|   - subtract mean = ', avg, ' from whole record'
        ncomments = ncomments + 1
        write(comments(ncomments),'(a)') 
     :      '|     mean was specified.' 
      else
        write(*,'(a)') ' Not supposed to be here.'
      end if

      ncomments = ncomments + 1
      comments(ncomments) = '|   resulting in output file '//
     :      f_out(1:nc_f_out)

      ncomments = ncomments + 1
      comments(ncomments) = '|'

      itype = 0

      call SMCWrite(f_out(1:nc_f_out), precision,
     :                y1y2_int, char_head, itype, 
     :                    int_head, real_head, comments)

 
      deallocate (y1y2_int)

      go to 1000

9999  continue

* Close control and output files:

      close(nu_ctl)
      close(nu_sum)

      stop
      end
* ------------------------------------------------------------------- SMCIntY1Y2


      include '\forprogs\skip.for'
      include '\forprogs\skipcmnt.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\upstr.for'
      include '\forprogs\smc_npts.for'
      include '\forprogs\smcread.for'
      include '\forprogs\smcwrite.for'
      include '\forprogs\integrate_y.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\datetime.for'
      include '\forprogs\rmvtrend.for'
      include '\forprogs\mnmaxidx.for'
      include '\forprogs\dcdt.for'
      include '\forprogs\distaz.for'
      

