* ------------------ BEGIN SMCXCorr -----------------------------
      Program SMCXCorr

* Read a control file with info for 1 to 10 pairs of smc files.  Read
* the smc files and cross correlate them to produce a column file (actually, in
* CoPlot parlance, a wrapped ascii file) for time lags specified in the control
* file.

* NOTE: the sps must be the same for the two files being cross correlated!

* Here as a sample input file:

*Control file for SMCXCorr      ! first line
*Name of output file:
* smcxcorr.asc
*1.1st series,2.2nd series,3.column tag,3.tlag_min, tlag_max
* c:\chichidt\a800\wnt1u.smc
* c:\chichidt\a900\tcu1291u.smc
* wntt129_1                      ! up to 8 characters, starting in any column
* -30.0 30.0 4  ! tlag_min,tlag_max,ndec(+ lag of peak xcorr => ts 1 delayed)
* c:\chichidt\a800\wnt2u.smc
* c:\chichidt\a900\tcu1292u.smc
* wntt129_2                      ! up to 8 characters, starting in any column
* -30.0 30.0 4  ! tlag_min,tlag_max,ndec(+ lag of peak xcorr => ts 1 delayed)
* c:\chichidt\a800\wnt3u.smc
* c:\chichidt\a900\tcu1293u.smc
* wntt129_3                      ! up to 8 characters, starting in any column
* -30.0 30.0 4  ! tlag_min,tlag_max,ndec(+ lag of peak xcorr => ts 1 delayed)
*stop

* Dates: 01/18/01 - Written by D. Boore, based on SMC2ASC
*        10/17/07 - Transparent change to user needed because smcread can
*                   read two input formats


      parameter(nts_max = 10)
      real sps(nts_max)
      real tlag_min(nts_max), tlag_max(nts_max)
      integer nc(nts_max)

      real ts_1(:,:), ts_2(:,:), c(:,:), tlag(:,:)
      allocatable :: ts_1, ts_2, c, tlag

      integer npts_1(nts_max), npts_2(nts_max), ndec(nts_max)
      real real_head(50)
      integer int_head(48), nc_f_ts_1(nts_max), nc_f_ts_2(nts_max)
      character char_head(11)*80, comments(500)*80
      character f_ctl*40, f_out*40, buf*40,
     :  colhead(nts_max)*11, xcorr_c(nts_max)*11, idtag*80
      character f_ts_1(nts_max)*80, f_ts_2(nts_max)*80
      logical f_ctl_exist, f_ts_exist

* NOTE: need f_ts_1, f_ts_2 to be arrays because need to read npts in each
* for the dynamic array allocation before actually reading the data from each.

*      npts_max = 100000            ! depends on dimensions statement
      

* Get name of input file:
      f_ctl_exist = .false.
      do while (.not. f_ctl_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr to quit): '
        read(*, '(a)') f_ctl
        if (f_ctl(1:4) .eq. '    ') stop
        inquire(file=f_ctl, exist=f_ctl_exist)
        if (.not. f_ctl_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do

* Open file 
      call get_lun(nu_ctl)      
      open(unit=nu_ctl, file=f_ctl, status='unknown')

* Read output file name:
*   Skip text lines:
      call skip(nu_ctl,2)
      f_out = ' '
      read(nu_ctl, '(a)') f_out

* Loop over sets of time series:
*  Skip text lines:
      call skip(nu_ctl,1)

      nts = 0

1000  continue

* Read the first file name:
        buf = ' '
        read(nu_ctl, '(a)') buf
        call upstr(buf)
        call trim_c(buf,nc_buf)

* Check for "STOP":
        if (buf(1:4) .eq. 'STOP') go to 2000

* Check for nts > max number
        if (nts + 1 .gt. nts_max) then
           write(*,*)
           write(*, '(a   )') 
     :    ' nts+1 > nts_max; ignoring remaining ts'
           write(*,*)
           go to 2000
        end if

* Passed tests; proceed
        nts = nts + 1
        f_ts_1(nts) = ' '
        f_ts_1(nts) = buf(1:nc_buf)
        nc_f_ts_1(nts) = nc_buf
        f_ts_2(nts) = ' '
        read(nu_ctl, '(a)') f_ts_2(nts)
        call upstr(f_ts_2(nts))
        call trim_c(f_ts_2(nts),nc_f_ts_2(nts))

        inquire(file=f_ts_1(nts), exist=f_ts_exist)
        if (.not. f_ts_exist) then
          write(*,'(3a)') ' ****** FILE ', f_ts_1(nts), 
     :          ' DOES NOT EXIST; QUITTING!!! ******* '
          close(nu_out)
          close(nu_ctl)
          stop
        end if

        inquire(file=f_ts_2(nts), exist=f_ts_exist)
        if (.not. f_ts_exist) then
          write(*,'(3a)') ' ****** FILE ', f_ts_2(nts), 
     :          ' DOES NOT EXIST; QUITTING!!! ******* '
          close(nu_out)
          close(nu_ctl)
          stop
        end if


* Read idtag:
        buf = ' '
        read(nu_ctl, '(a)') buf
        call RCC(1,buf,40,idtag,nc_id,ibgn,iend)
        if (nc_id .gt. 8) nc_id = 8
        write(*,*)
        write(*,'(2a)') '  idtag = ', idtag(1:nc_id)
        
* Read tlag_min, tlag_max, ndec:
        read(nu_ctl,*)  tlag_min(nts), tlag_max(nts), ndec(nts)

        write(*,'(2a)') '  f_ts_1 = ', f_ts_1(nts)
        write(*,'(2a)') '  f_ts_2 = ', f_ts_2(nts)
        write(*,'(a)') 
     :      ' nts, tlag_min(nts), tlag_max(nts), ndec(nts) = '
        write(*,*)
     :        nts, tlag_min(nts), tlag_max(nts), ndec(nts)
 
* Set up the column headings:
        colhead(nts) = ' '
        colhead(nts) = 'ts:'//idtag(1:nc_id)//'........'
        write(*,'(2a)') ' colhead = ', colhead(nts)

* Get number of points on this loop:

        call smc_nsps(f_ts_1(nts),npts_1(nts), sps_1)
        call smc_nsps(f_ts_2(nts),npts_2(nts), sps_2)

        if (sps_1 .ne. sps_2) then
          write(*,'(a,1p2(1x,e10.3))') 
     :     '  ERROR!!! SPS not equal, Quitting.  sps_1, sps_2 = ',
     :      sps_1, sps_2
          close(unit=nu_ctl)
        end if

        sps(nts) = sps_1

        write(*,'(a,2(1x,i6))') ' nts, npts_1(nts) = ',nts,npts_1(nts)
        write(*,'(a,2(1x,i6))') ' nts, npts_2(nts) = ',nts,npts_2(nts)

        go to 1000  ! loop back for another file

2000  continue

      close(unit=nu_ctl)

* Figure out largest value of npts_1, npts_2, 
* and allocate the arrays:
      call imnmax(npts_1,1,nts,1,imin_1,imax_1) 
      call imnmax(npts_2,1,nts,1,imin_2,imax_2) 
      imax = max0(imax_1, imax_2)
     
      write(*,*)
      write(*, '(a, 2(1x,i5))') ' nts,nts_max = ', nts, nts_max
      write(*, '(a, 2(1x,i5))') 
     :                ' imin_1, imax_1 = ', imin_1, imax_1
      write(*, '(a, 2(1x,i5))') 
     :                ' imin_2, imax_2 = ', imin_2, imax_2
      write(*, '(a, 2(1x,i5))') ' imax = ', imax

      write(*,'(a)') ' Before allocate'
      allocate( ts_1(imax,nts_max), ts_2(imax,nts_max))
      write(*,'(a)') ' After first allocate'
      allocate( c(imax,nts_max), tlag(imax,nts_max))
      write(*,'(a)') ' After second allocate'
      
* Now read the data from the files:
     
      do i = 1, nts
        tskip = 0.0
        tlength = 100000.0
        write(*,'(a)') ' Before SMCRead'
        call SMCRead(f_ts_1(i), tskip, tlength,
     :               ts_1(1,i),  
     :               char_head, int_head, real_head, comments)
        call SMCRead(f_ts_2(i), tskip, tlength,
     :               ts_2(1,i),  
     :               char_head, int_head, real_head, comments)
        write(*,'(a)') ' After SMCRead'


* Do the cross-correlation:
        write(*,'(a)') ' Before call XCorr'
        call XCorr(ts_1(1,i),npts_1(i),
     :             ts_2(1,i),npts_2(i),
     :                c(1,i),tlag(1,i),nc(i),
     :              tlag_min(i),tlag_max(i),sps(i),ndec(i))
        write(*,'(a)') ' After call Xcorr'

      end do 

* Write output:

* Open output file:
      call get_lun(nu_out)      
      open(unit=nu_out, file=f_out, status='unknown')

      write(nu_out, '(a,1x,i2)') 
     :     ' Number of variables = ', 2*nts
      write(nu_out, '(a,1x,i3)') 
     :     ' Variable names start on line # ', 4+2*nts+1
      write(nu_out, '(a,1x,i3)') 
     :     ' Data starts on line # ', 4+2*nts+2

      write(nu_out, '(2a)') ' File with input parameters: ', f_ctl
 
      do i = 1, nts
        write(nu_out, '(3a)') 
     :    ' Files to be correlated: ', 
     :    f_ts_1(i)(1:nc_f_ts_1(i)), f_ts_2(i)(1:nc_f_ts_2(i))
        write(nu_out, '(3x, a, 1p2(1x,e10.3))') 
     :    ' tlag_min, tlag_max = ', tlag_min(i), tlag_max(i)
      end do

      write(nu_out, '(10(1x,3x,a4,1x,a11))') 
     :    ('time', colhead(j), j = 1, nts)
      
* Find new jmax:
      call imnmax(nc,1,nts,1,jmin,jmax) 

      do j = 1, jmax

* Account for unequal number of points:
        do i = 1, nts
          if (j .le. nc(i)) then              ! fill output string with value
            write(xcorr_c(i), '(1pe11.4)') c(j,i)
* Note: best if could use blanks, but CoPlot does not import this
* correctly as wrapped ascii.  It would import properly if I made
* a column file, but I am reluctant to strip off the informative header lines 
*          else                                  ! fill with blanks
*            xcorr_c(i) = ' '
          else                                  ! fill with very large number
            write(xcorr_c(i), '(1pe11.4)') 1.0e37
          end if
        end do
        write(nu_out, '(10(1x,f7.3,1x,a))')
     :    (tlag(j,i),xcorr_c(i), i = 1, nts)
      end do

      deallocate(ts_1, ts_2, c, tlag)

      close(unit=nu_out)

      stop
      end
* ------------------ END SMCCORRL -----------------------------

      include '\forprogs\SMCRead.for'
      include '\forprogs\get_lun.for'
      include '\forprogs\imnmax.for'
      include '\forprogs\skip.for'
      include '\forprogs\trim_c.for'
      include '\forprogs\upstr.for'
      include '\forprogs\rc_subs.for'
      include '\forprogs\smc_nsps.for'
      include '\forprogs\xcorr.for'

